#ifndef __CONSTRAINTS_H__
#define __CONSTRAINTS_H__

#include <string>
#include <vector>

#include "Disposable.h"
#include "Expression.h"

namespace Inter4ql {

	class Fact;

	class QuantifierEntry {
	public:
		QuantifierEntry(std::string type, std::string id, std::string domain)
		    : type_(type),
		      id_(id),
		      domain_(domain) {};
		std::string get_type() { return type_; };
		std::string get_id() { return id_; };
		std::string get_domain() { return domain_; };
	private:
		std::string type_;
		std::string id_;
		std::string domain_;
	};

	class ConstraintEntry {
	public:
		ConstraintEntry(std::vector<QuantifierEntry*>* q, Expression* body, std::vector<Inter4ql::Value*>* values)
		    : quantifiers_(q),
		      body_(body),
		      values_(values) {};
		~ConstraintEntry() {
			if(!Disposable::errorMode) {
				if (this->quantifiers_) {
					for(std::vector<QuantifierEntry*>::iterator it = quantifiers_->begin(); it != quantifiers_->end(); it++) {
						delete (*it);
					}
					delete this->quantifiers_;
				}

				if (this->values_) {
					for(std::vector<Inter4ql::Value*>::iterator it = values_->begin(); it != values_->end(); it++) {
						delete (*it);
					}
					delete this->values_;
				}

				if (this->body_)
					delete this->body_;
			}
		}
		std::vector<QuantifierEntry*>* get_quantifiers() { return quantifiers_; };
		Expression* get_body() { return body_; };
		std::vector<Inter4ql::Value*>* get_values() { return values_; };
	private:
		std::vector<QuantifierEntry*>* quantifiers_;
		Expression* body_;
		std::vector<Inter4ql::Value *>* values_;
	};

	typedef std::vector<Expression*>* Ctraints;

	/** @brief Output class handles standard output of interpreter
	 */
	class Constraints {
	public:
		/** @brief constructor
		 *
		 * @param o standard output
		 * @param e error output
		 */
		Constraints(Ctraints _hard, Ctraints _soft);
		/** @brief destructor
		 */
		~Constraints();
		/** @brief getter for flexible constraints
		 *
		 * @return constraints list
		 */
		Ctraints get_soft_constraints();
		/** @brief getter for rigid constraints
		 *
		 * @return constraints list
		 */
		Ctraints get_hard_constraints();
	private:
		Ctraints soft;
		Ctraints hard;
	};
}

#endif
