#include "Disposable.h"

#include <cassert>
#include <iostream>
#include <vector>

#include "Application.h"
#include "Program.h"
#include "ModuleLocal.h"
#include "ModuleGroup.h"
#include "ModuleBelief.h"
#include "ModuleAction.h"
#include "ModuleProblem.h"
#include "Constraints.h"
#include "Domain.h"
#include "Relation.h"
#include "Rule.h"
#include "Fact.h"
#include "Value.h"

namespace Inter4ql {
	bool Disposable::errorMode = false;

	// keep in sync with disp_type...
	const std::string names[] = {"PROGRAM", "MODULE_LOCAL_VECTOR", "MODULE_LOCAL", "MODULE_GROUP", "MEMBERS_VECTOR",
			"DOMAIN_VECTOR", "DOMAIN_suf", "RELATION_VECTOR", "RELATION", "RULE_VECTOR", "RULE", "FACT_VECTOR",
			"FACT", "FACT_VECTOR_VECTOR", "VALUE_VECTOR", "VALUE", "MODULE_BELIEF", "MODULE_ACTION", "MODULE_PROBLEM",
			"CONSTRAINTS", "CONSTRAINT_ENTRY", "QUANTIFIER_ENTRY", "QUANTIFIERS_VECTOR", "CONSTRAINT_ENTRY_VECTOR,"
			"EXPRESSION_VECTOR", "EXPRESSION"};

	Disposable::Disposable() {
		this->objects = new std::map<disp_type::type, std::vector<void*>*>();
		errorMode = false;
		this->debug = false;
	}

	Disposable::~Disposable() {
		for(std::map<disp_type::type, std::vector<void*>*>::iterator it = this->objects->begin(); it != this->objects->end(); it++) {
			delete (it->second);
		}
		this->objects->clear();
		delete this->objects;
	}

	void Disposable::clear() {
		errorMode = true;

		for(std::map<disp_type::type, std::vector<void*>*>::iterator it = this->objects->begin(); it != this->objects->end(); it++) {
			disp_type::type currentType = it->first;

#define CASE_DELETE(name, type) \
	case disp_type::name: if(Application::app_instance->debug_disposable) {std::cout << "DELETE " #name << "(" << *it1 << ")"<< std::endl;}; delete (type*) (*it1); break;

				for(std::vector<void*>::iterator it1 = it->second->begin(); it1 != it->second->end(); it1++) {
					switch(currentType) {
						CASE_DELETE(PROGRAM, Program)
						CASE_DELETE(MODULE_LOCAL_VECTOR, std::vector<ModuleLocal*>)
						CASE_DELETE(MODULE_LOCAL, ModuleLocal)
						CASE_DELETE(MODULE_GROUP, ModuleGroup)
						CASE_DELETE(MEMBERS_VECTOR, std::vector<std::string>)
						CASE_DELETE(DOMAIN_VECTOR, std::vector<Domain*>)
						CASE_DELETE(DOMAIN_suf, Domain)
						CASE_DELETE(RELATION_VECTOR, std::vector<Relation*>)
						CASE_DELETE(RELATION, Relation)
						CASE_DELETE(RULE_VECTOR, std::vector<Rule*>)
						CASE_DELETE(RULE, Rule)
						CASE_DELETE(FACT_VECTOR_VECTOR, std::vector<std::vector<Fact*>*>)
						CASE_DELETE(FACT_VECTOR, std::vector<Fact*>)
						CASE_DELETE(FACT, Fact)
						CASE_DELETE(VALUE, Value)
						CASE_DELETE(VALUE_VECTOR, std::vector<Value*>)
						CASE_DELETE(MODULE_BELIEF, ModuleBelief)
						CASE_DELETE(MODULE_ACTION, ModuleAction)
						CASE_DELETE(MODULE_PROBLEM, ModuleProblem)
						CASE_DELETE(CONSTRAINTS, Constraints)
						CASE_DELETE(CONSTRAINT_ENTRY, ConstraintEntry)
						CASE_DELETE(QUANTIFIER_ENTRY, QuantifierEntry)
						CASE_DELETE(QUANTIFIERS_VECTOR, std::vector<QuantifierEntry*>)
						CASE_DELETE(CONSTRAINT_ENTRY_VECTOR, std::vector<ConstraintEntry*>)
						CASE_DELETE(EXPRESSION_VECTOR, std::vector<Expression*>)
						CASE_DELETE(EXPRESSION, Expression)
						default: std::cout << "Error while deleting objects: unknown object!" << std::endl; break;
					}
				}

				delete (it->second);
		}

		this->objects->clear();

		errorMode = false;
	}

	void Disposable::add(void* address, disp_type::type typeToAdd) {
		std::map<disp_type::type, std::vector<void*>*>::iterator it = this->objects->find(typeToAdd);

		if(Application::app_instance->debug_disposable) {
			std::cout << "Adding type " << names[typeToAdd] << std::endl;
		}

		if(it == this->objects->end()) {
			std::vector<void*>* temp = new std::vector<void*>();
			temp->push_back(address);
			objects->insert(std::make_pair(typeToAdd, temp));
		}  else {
			it->second->push_back(address);
		}

		return;
	}

	void Disposable::remove(void* address, disp_type::type typeToAdd) {
		std::map<disp_type::type, std::vector<void*>*>::iterator it = this->objects->find(typeToAdd);

		if(Application::app_instance->debug_disposable) {
			std::cout << "Removing type " << names[typeToAdd] << std::endl;
		}

		if(it == this->objects->end()) {
			// this should never occur!
			assert(false);
		}  else {
			for(std::vector<void*>::iterator iter = it->second->begin(); iter != it->second->end();) {
				if (*iter == address) {
					iter = it->second->erase(iter);
				} else {
					iter++;
				}
			}
		}

		return;
	}

}
