#ifndef __EXPRESSSION_CC__
#define __EXPRESSSION_CC__

#include "Disposable.h"
#include "Expression.h"
#include "Fact.h"

namespace Inter4ql {

	Expression::Expression(expression_type::type type, Expression* lexpr, Expression* rexpr, Fact* term)
		: my_type(type)
		, my_lexpr(lexpr)
		, my_rexpr(rexpr)
		, my_elseexpr(nullptr)
		, my_term(term)
		, my_quantified_entry(nullptr)
		, should_delete_term(true)
		, simple_relation_flag(false)
	{ }

	// expression type used for composite actions
	Expression::Expression(expression_type::type type, Expression* lexpr, Expression* rexpr, Expression* elseexpr)
		: my_type(type)
		, my_lexpr(lexpr)
		, my_rexpr(rexpr)
		, my_elseexpr(elseexpr)
		, my_term(nullptr)
		, my_quantified_entry(nullptr)
		, should_delete_term(false)
		, simple_relation_flag(false)
	{ }

	Expression::Expression(ConstraintEntry* qe)
		: my_type(expression_type::QUANTIFIED)
		, my_lexpr(nullptr)
		, my_rexpr(nullptr)
		, my_elseexpr(nullptr)
		, my_term(nullptr)
		, my_quantified_entry(qe)
		, should_delete_term(true)
		, simple_relation_flag(false)
	{ }

	Expression::~Expression() {
		if(!Disposable::errorMode) {
			if (my_term && should_delete_term) {
				for(std::vector<Value*>::iterator it2 = my_term->get_params()->begin(); it2 != my_term->get_params()->end(); it2++) {
					delete (*it2);
				}
				delete my_term;
			}
			if (my_lexpr)
				delete my_lexpr;
			if (my_rexpr)
				delete my_rexpr;
		}
	};

	expression_type::type Expression::get_type()
	{
		return my_type;
	}

	Expression* Expression::get_left_expression()
	{
		return my_lexpr;
	}

	Expression* Expression::get_right_expression()
	{
		return my_rexpr;
	}

	Expression* Expression::get_else_expression()
	{
		return my_elseexpr;
	}

	Fact* Expression::get_term()
	{
		return my_term;
	}

	ConstraintEntry* Expression::get_quantified_entry()
	{
		return my_quantified_entry;
	}

	void Expression::set_module(std::string s)
	{
		module = s;
	}

	std::string Expression::get_module()
	{
		return module;
	}

	void Expression::set_delete_term(bool value) {
		this->should_delete_term = value;
	}

	void Expression::set_simple_relation_flag(bool value) {
		this->simple_relation_flag = value;
	}

	bool Expression::is_simple_relation() {
		return this->simple_relation_flag;
	}
}

#endif
