%require "3.0.2"
%skeleton "lalr1.cc"
%defines
%locations
%define api.namespace{Inter4ql}
%define parser_class_name{LineParser}
/*
%define api.namespace{Inter4ql}
%define api.parser.class {LineParser}
*/
%define parse.error verbose
%parse-param	{ Inter4ql::Scanner &scanner }
%parse-param	{ Inter4ql::parser_line &data }
%lex-param	  { Inter4ql::Scanner &scanner }

%code requires {

#include <vector>
#include <iostream>
#include "Value.h"
#include "Domain.h"
#include "Fact.h"
#include "Relation.h"
#include "Disposable.h"
#include "Expression.h"
#include "BooleanExpression.h"
#include "Rule.h"
#include "Constraints.h"

	namespace Inter4ql {
		class Scanner;
		class ModuleGeneric;
		class ModuleLocal;
		class ModuleGroup;
		class ModuleBelief;
		class ModuleAction;
		class ModuleProblem;
		class Program;
	}
}

%code {

#include "position.hh"
#include "Functions.h"
#include "Exception.h"
#include "Program.h"
#include "ModuleGeneric.h"
#include "ModuleLocal.h"
#include "ModuleGroup.h"
#include "ModuleBelief.h"
#include "ModuleAction.h"
#include "ModuleProblem.h"

	static int yylex(
			Inter4ql::LineParser::semantic_type *yylval,
			Inter4ql::LineParser::location_type *yylloc,
			Inter4ql::Scanner &scanner);

	extern std::vector<Inter4ql::Value*>* toDelete;
	extern Inter4ql::Disposable* toDeleteAll;
	bool debug = false;

	void addToDelete(void* address, Inter4ql::disp_type::type typeToAdd) {
		if(!toDeleteAll) toDeleteAll = new Inter4ql::Disposable();

		toDeleteAll->add(address, typeToAdd);
	}

	void removeFromDelete(void* address, Inter4ql::disp_type::type typeToAdd) {
		if(!toDeleteAll) toDeleteAll = new Inter4ql::Disposable();

		toDeleteAll->remove(address, typeToAdd);
	}
}

%union {
	variable_type::type var_type;

	Disposable* toDelete;

	Value *value;
	std::vector<Inter4ql::Value *> *values;

	Domain *domain;
	std::vector<Domain *> *domains;

	Fact *fact;
	std::vector<Fact *> *facts;

	Relation *relation;
	std::vector<Relation*> *relations;

	ModuleLocal *module;
	ModuleGroup *group;
	std::vector<ModuleGeneric *> *modules;

	std::vector<std::string> *members;

	ModuleBelief *beliefs;
	Constraints *constraints;
	ConstraintEntry *single_constraint;
	std::vector<Expression*> *hard_soft_constraints;
	std::vector<QuantifierEntry*> *quantifiers;

	ModuleAction *action;

	ModuleProblem* problem;

	Rule *rule;
	std::vector<Rule *> *rules;
	std::vector<std::vector<Fact*>*> *rule_primitive;

	Expression* expression;
	BooleanExpression* bexpression;
	std::vector<BooleanExpression*>* bexpressions;
	BooleanExpression::Comparison_Type comparison_type;

	struct parser_line *line;
	Program *program;
}

%token MODULE MODULES GROUP CONSTRAINTS FLEXIBLE RIGID BELIEFS AS MEMBERS WORLDS DOMAINS RELATIONS RULES FACTS END IMPORT
%token IMPL COMMA DOT COLON OPEN_PARENTHESIS CLOSE_PARENTHESIS AND OR EXCLAMATION MINUS PLUS DIVIDE MULTIPLY TILDA SEMICOLON
%token ASSIGN ARROW IF_ARROW EQ NEQ GT GTE LT LTE BEL FORALL EXISTS
%token ACTION PRECONDITIONS POSTCONDITIONS ADD REMOVE COMPOSITE PARALLEL SEQUENTIAL
%token PROBLEM PROBLEMS ACTIONS GOAL MAX_DEPTH
%token SHARP_ALL SHARP_TRUE SHARP_FALSE SHARP_INCONSISTENT SHARP_UNKNOWN FALSE_WHEN TRUE_WHEN INCONSISTENT_WHEN
%token <value> VARIABLE ID STRING INTEGER REAL DATE DATE_TIME LOGIC INVALID
%token IN OPEN_CURLY CLOSE_CURLY OPEN_BRACKETS CLOSE_BRACKETS
%token CONST_T STRING_T INTEGER_T REAL_T LOGIC_T DATE_T DATE_TIME_T
%token HEURISTICS RANDOM COMPOSITE_FIRST GOAL_PROBING NONE DISALLOW_FAILED_PRECONDITIONS INCREASE_RATING_ONLY STRICT_INCREASE_RATING_ONLY
%token PRINT CLEAR EXECUTE
%token DEBUG ENABLE DISABLE HELP
%token EXIT

%left COMMA AND OR AS SEQUENTIAL PARALLEL DIVIDE
%right UNOT ARROW IF_ARROW

%type <var_type> type

%type <action> action

%type <problem> problem

%type <value> value argument as_expression heuristics
%type <values> values arguments arguments_empty logic_list oneelem_logic_list

%type <domain> domain_declaration
%type <domains> parameters domain_declarations domains

%type <fact> fact_declaration rule_head relation_in_rule relation_in_rule_curly bel
%type <facts> facts fact_declarations

%type <relation> relation_declaration
%type <relations> relation_declarations relations

%type <rule> rule_declaration
%type <rules> rule_declarations rules to_add to_remove
%type <rule_primitive> relations_list

%type <module> module
%type <modules> modules

%type <group> group
%type <members> members worlds members_list string_list heuristics_list heuristics_list_internal

%type <expression> expression expression_entry expression_actions simple_relation query preconditions composite
%type <bexpression> true_when false_when inconsitent_when bexpression bexp1 bexp2 bexp3 bexp4 bexp5
%type <bexpressions> when_info
%type <comparison_type> COMPARISON

%type <constraints> constraints
%type <hard_soft_constraints> soft_constraints hard_constraints constraints_entry_list
%type <quantifiers> quantifier_list
%type <beliefs> belief

%type <program> program
%type <line> line

%destructor {
	if(toDelete) {
		for(std::vector<Inter4ql::Value*>::iterator it = toDelete->begin(); it != toDelete->end(); it++) {
			delete (*it);
		}
		delete toDelete;
	}
	toDelete = 0;

	if(toDeleteAll) {
		delete toDeleteAll;
	}
	toDeleteAll = 0;
} line

%%

line:	   	program
			{
				if(debug) std::cout << "Parsed program" << std::endl;
				data.type = line_type::PROGRAM;
				data.program = $1;
				data.rule = 0;
				$$ = 0;
			}
	|       query DOT /*relation_in_rule DOT*/
			{
				if(debug) std::cout << "Parsed relation_in_rule DOT" << std::endl;
				data.type = line_type::RULE;
				data.program = 0;
				data.rule = 0;
				data.expression = $1;
				$$ = 0;
			}
	|       PRINT ID DOT
			{
				if(debug) std::cout << "Parsed PRINT ID DOT" << std::endl;
				data.type = line_type::PRINT;
				data.value = $2->get_value_string();
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       EXECUTE ID DOT
			{
				if(debug) std::cout << "Parsed EXECUTE ID DOT" << std::endl;
				data.type = line_type::EXECUTE;
				data.value = $2->get_value_string();
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       IMPORT STRING DOT
			{
				if(debug) std::cout << "Parsed IMPORT STRING DOT" << std::endl;
				data.type = line_type::IMPORT;
				data.value = $2->get_value_string();
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       DEBUG DOT
			{
				if(debug) std::cout << "Parsed DEBUG DOT" << std::endl;
				data.type = line_type::DEBUG;
				data.value = "";
				data.bool_value = false;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       DEBUG STRING ENABLE DOT
			{
				if(debug) std::cout << "Parsed DEBUG STRING ENABLE DOT" << std::endl;
				data.type = line_type::DEBUG;
				data.value = $2->get_value_string();
				data.bool_value = true;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       DEBUG STRING DISABLE DOT
			{
				if(debug) std::cout << "Parsed DEBUG STRING DISABLE DOT" << std::endl;
				data.type = line_type::DEBUG;
				data.value = $2->get_value_string();
				data.bool_value = false;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       HELP
			{
				if(debug) std::cout << "Parsed IMPORT STRING DOT" << std::endl;
				data.type = line_type::HELP;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       HELP DOT
			{
				if(debug) std::cout << "Parsed HELP DOT" << std::endl;
				data.type = line_type::HELP;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       EXIT
			{
				if(debug) std::cout << "Parsed EXIT" << std::endl;
				data.type = line_type::QUIT;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       EXIT DOT
			{
				if(debug) std::cout << "Parsed EXIT DOT" << std::endl;
				data.type = line_type::QUIT;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       MODULES DOT
			{
				if(debug) std::cout << "Parsed MODULES DOT" << std::endl;
				data.type = line_type::MODULES;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       ACTIONS DOT
			{
				if(debug) std::cout << "Parsed ACTIONS DOT" << std::endl;
				data.type = line_type::ACTIONS;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       PROBLEMS DOT
			{
				if(debug) std::cout << "Parsed PROBLEMS DOT" << std::endl;
				data.type = line_type::PROBLEMS;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
	|       CLEAR DOT
			{
				if(debug) std::cout << "Parsed CLEAR DOT" << std::endl;
				data.type = line_type::CLEAR;
				data.rule = 0;
				data.program = 0;
				$$ = 0;
			}
;

program:	modules
			{
				if(debug) std::cout << "Parsed modules" << std::endl;
				$$ = new Program($1);
				addToDelete((void*) $$, Inter4ql::disp_type::PROGRAM);
			}
;
modules:	/* empty */
			{
				if(debug) std::cout << "Parsed empty(modules)" << std::endl;
				$$ = new std::vector<ModuleGeneric *>();
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_LOCAL_VECTOR);
			}
	|       modules module
			{
				if(debug) std::cout << "Parsed modules module" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
	|      modules group
			{
				if(debug) std::cout << "Parsed groups group" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
	|      modules belief
			{
				if(debug) std::cout << "Parsed belief module" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
	|      modules action
			{
				if(debug) std::cout << "Parsed action module" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
	|      modules problem
			{
				if(debug) std::cout << "Parsed problem module" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
;
module:		MODULE ID COLON constraints domains relations rules facts END DOT
			{
				if(debug) std::cout << "Parsed MODULE ID COLON domains relstions rules facts END DOT" << std::endl;
				$$ = new ModuleLocal($2->get_value_string(), $4, $5, $6, $7, $8);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_LOCAL);
			}
;
belief:		BELIEFS ID COLON constraints worlds END DOT
			{
				if(debug) std::cout << "Parsed BELIEF ID COLON constraints worlds END DOT" << std::endl;
				$$ = new ModuleBelief($2->get_value_string(), $4, $5);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_BELIEF);
			}
;
action:		ACTION ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS COLON preconditions to_add to_remove END DOT
			{
				if(debug) std::cout << "Parsed ACTION ID COLON PRECONDITIONS preconditions to_add to_remove END DOT" << std::endl;
				$$ = new ModuleAction($2->get_value_string(), $4, $7, $8, $9, nullptr);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_ACTION);
			}
	|	ACTION ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS COLON preconditions composite END DOT
			{
				if(debug) std::cout << "Parsed ACTION ID COLON PRECONDITIONS preconditions composite END DOT" << std::endl;
				$$ = new ModuleAction($2->get_value_string(), $4, $7, nullptr, nullptr, $8);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_ACTION);
			}
	|	ACTION ID OPEN_PARENTHESIS arguments_empty CLOSE_PARENTHESIS COLON preconditions to_add to_remove END DOT
			{
				if(debug) std::cout << "Parsed ACTION ID COLON PRECONDITIONS preconditions to_add to_remove END DOT" << std::endl;
				$$ = new ModuleAction($2->get_value_string(), $4, $7, $8, $9, nullptr);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_ACTION);
			}
	|	ACTION ID OPEN_PARENTHESIS arguments_empty CLOSE_PARENTHESIS COLON preconditions composite END DOT
			{
				if(debug) std::cout << "Parsed ACTION ID COLON PRECONDITIONS preconditions composite END DOT" << std::endl;
				$$ = new ModuleAction($2->get_value_string(), $4, $7, nullptr, nullptr, $8);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_ACTION);
			}
;
;
problem:	PROBLEM ID COLON BELIEFS COLON string_list ACTIONS COLON string_list GOAL COLON expression MAX_DEPTH COLON INTEGER HEURISTICS COLON heuristics_list END DOT
			{
				if(debug) std::cout << "Parsed PROBLEM ID COLON BELIEFS COLON string_list ACTIONS COLON string_list GOAL COLON expression MAX_DEPTH COLON INTEGER HEURISTICS COLON heuristics_list END DOT" << std::endl;
				$$ = new ModuleProblem($2->get_value_string(), $6, $9, $12, $15->get_value_int(), $18);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_PROBLEM);
			}
;
group:		GROUP ID COLON domains relations when_info members END DOT
			{
				if(debug) std::cout << "Parsed GROUP ID COLON domains relations members END DOT" << std::endl;
				BooleanExpression * i = $6->at(0);
				BooleanExpression * t = $6->at(1);
				BooleanExpression * f = $6->at(2);
				$$ = new ModuleGroup($2->get_value_string(), $4, $5, $7);
				$$->add_whens(t, f, i);
				addToDelete((void*) $$, Inter4ql::disp_type::MODULE_GROUP);
			}
;
preconditions:	/* empty */
	     		{
				if(debug) std::cout << "Parsed empty(preconditions)" << std::endl;

				std::vector<Value*>* arguments = new std::vector<Value *>();
				addToDelete((void*) arguments, Inter4ql::disp_type::VALUE_VECTOR);
				Value* temp = new Value("true");
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				arguments->push_back(temp);
				Fact* rule_fact  = new Fact("math", "logic_value", arguments);
				addToDelete((void*) rule_fact, Inter4ql::disp_type::FACT);
				Expression* e = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, rule_fact);
				addToDelete((void*) e, Inter4ql::disp_type::EXPRESSION);

				$$ = e;
			}
	|	PRECONDITIONS COLON expression DOT
			{
				if(debug) std::cout << "Parsed expression(preconditions)" << std::endl;
				$$ = $3;
			}
;
to_add: /* empty */
			{
				if(debug) std::cout << "Parsed empty(to_add)" << std::endl;
				$$ = new std::vector<Rule *>();
				addToDelete((void*) $$, Inter4ql::disp_type::RULE_VECTOR);
			}
	|	ADD COLON rule_declarations
			{
				if(debug) std::cout << "Parsed ADD COLON rules" << std::endl;
				$$ = $3;
			}
;
to_remove:	/* empty */
			{
				if(debug) std::cout << "Parsed empty(to_remove)" << std::endl;
				$$ = new std::vector<Rule *>();
				addToDelete((void*) $$, Inter4ql::disp_type::RULE_VECTOR);
			}
	|	REMOVE COLON rule_declarations
			{
				if(debug) std::cout << "Parsed REMOVE COLON rules" << std::endl;
				$$ = $3;
			}
;
composite:	COMPOSITE COLON expression_actions
			{
				if(debug) std::cout << "Parsed COMPOPSITE COLON modules" << std::endl;
				$$ = $3;
			}
;
expression_actions:
		expression
			{
				if(debug) std::cout << "Parsed simple_relation (actions)" << std::endl;
				$$ = $1;
			}
	|	expression_actions PARALLEL expression_actions
			{
				if(debug) std::cout << "Parsed actions PARALLEL actions" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::PARALLEL, $1, $3);
			}
	|	OPEN_PARENTHESIS expression_actions PARALLEL expression_actions CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed actions PARALLEL actions" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::PARALLEL, $2, $4);
			}
	|	expression_actions SEQUENTIAL expression_actions
			{
				if(debug) std::cout << "Parsed actions SEQUENTIAL actions" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::SEQUENTIAL, $1, $3);
			}
	|	OPEN_PARENTHESIS expression_actions SEQUENTIAL expression_actions CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed actions SEQUENTIAL actions" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::SEQUENTIAL, $2, $4);
			}
	|	expression_entry IF_ARROW expression_actions DIVIDE expression_actions
			{
				if(debug) std::cout << "Parsed expression ARROW actions DIVIDE actions" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::IMPLIES, $1, $3, $5);
			}
	|	OPEN_PARENTHESIS expression_entry IF_ARROW expression_actions DIVIDE expression_actions CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed expression ARROW actions DIVIDE actions" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::IMPLIES, $2, $4, $6);
			}
;
constraints:	/* empty */
			{
				if(debug) std::cout << "Parsed empty(constraints)" << std::endl;
				$$ = new Constraints(NULL, NULL);
				addToDelete((void*) $$, Inter4ql::disp_type::CONSTRAINTS);
			}
	|	CONSTRAINTS COLON hard_constraints soft_constraints
			{
				if(debug) std::cout << "Parsed hard_constraints soft_constraints" << std::endl;
				$$ = new Constraints($3, $4);
				addToDelete((void*) $$, Inter4ql::disp_type::CONSTRAINTS);
			}
	|	CONSTRAINTS COLON soft_constraints hard_constraints
			{
				if(debug) std::cout << "Parsed soft_constraints hard_constraints" << std::endl;
				$$ = new Constraints($4, $3);
				addToDelete((void*) $$, Inter4ql::disp_type::CONSTRAINTS);
			}
	|	CONSTRAINTS COLON soft_constraints
			{
				if(debug) std::cout << "Parsed soft_constraints" << std::endl;
				$$ = new Constraints(NULL, $3);
				addToDelete((void*) $$, Inter4ql::disp_type::CONSTRAINTS);
			}
	|	CONSTRAINTS COLON hard_constraints
			{
				if(debug) std::cout << "Parsed hard_constraints" << std::endl;
				$$ = new Constraints($3, NULL);
				addToDelete((void*) $$, Inter4ql::disp_type::CONSTRAINTS);
			}
;
hard_constraints:
		RIGID COLON constraints_entry_list
			{
				if(debug) std::cout << "Parsed HARD COLON constraints_entry_list" << std::endl;
				$$ = $3;
			}
;
soft_constraints:
		FLEXIBLE COLON constraints_entry_list
			{
				if(debug) std::cout << "Parsed SOFT COLON constraints_entry_list" << std::endl;
				$$ = $3;
			}
;
constraints_entry_list:
		/* empty */
			{
				if(debug) std::cout << "Parsed empty(constraints_entry_list)" << std::endl;
				$$ = new std::vector<Expression*>();
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION_VECTOR);
			}
	|	constraints_entry_list expression DOT
			{
				if(debug) std::cout << "Parsed constraints_entry_list expression" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
;
quantifier_list:
		EXISTS VARIABLE COLON ID
			{
				if(debug) std::cout << "Parsed exists(quantifier_list)" << std::endl;
				$$ = new std::vector<QuantifierEntry*>();
				addToDelete((void*) $$, Inter4ql::disp_type::QUANTIFIERS_VECTOR);
				QuantifierEntry* qe = new QuantifierEntry("exists", $2->get_value_string(), $4->get_value_string());
				$$->push_back(qe);
				addToDelete((void*) qe, Inter4ql::disp_type::QUANTIFIER_ENTRY);
			}
	|	FORALL VARIABLE COLON ID
			{
				if(debug) std::cout << "Parsed forall(quantifier_list)" << std::endl;
				$$ = new std::vector<QuantifierEntry*>();
				addToDelete((void*) $$, Inter4ql::disp_type::QUANTIFIERS_VECTOR);
				QuantifierEntry* qe = new QuantifierEntry("forall", $2->get_value_string(), $4->get_value_string());
				$$->push_back(qe);
				addToDelete((void*) qe, Inter4ql::disp_type::QUANTIFIER_ENTRY);
			}
	|	quantifier_list FORALL VARIABLE COLON ID
			{
				if(debug) std::cout << "Parsed quantifier_list FORALL VARIABLE COLON ID" << std::endl;
				QuantifierEntry* qe = new QuantifierEntry("forall", $3->get_value_string(), $5->get_value_string());
				$1->push_back(qe);
				addToDelete((void*) qe, Inter4ql::disp_type::QUANTIFIER_ENTRY);
				$$ = $1;
			}
	|	quantifier_list EXISTS VARIABLE COLON ID
			{
				if(debug) std::cout << "Parsed quantifier_list EXISTS VARIALBE COLON ID" << std::endl;
				QuantifierEntry* qe = new QuantifierEntry("exists", $3->get_value_string(), $5->get_value_string());
				$1->push_back(qe);
				addToDelete((void*) qe, Inter4ql::disp_type::QUANTIFIER_ENTRY);
				$$ = $1;
			}
;
when_info:	inconsitent_when true_when false_when
			{
				$$ = new std::vector<BooleanExpression*>;
				$$->push_back($1);
				$$->push_back($2);
				$$->push_back($3);
			}
;
inconsitent_when:
		/* empty */
			{
				if(debug) std::cout << "Parsed empty(inconsitent_when)" << std::endl;
				$$ = 0;
			}
	|	INCONSISTENT_WHEN COLON bexpression
			{
				if(debug) std::cout << "Parsed INCONSISTENT_WHEN COLON bexpression" << std::endl;
				$$ = $3;
			}
;
true_when:	/* empty */
			{
				if(debug) std::cout << "Parsed empty(true_when)" << std::endl;
				$$ = 0;
			}
	|	TRUE_WHEN COLON bexpression
			{
				if(debug) std::cout << "Parsed TRUE_WHEN COLON bexpression" << std::endl;
				$$ = $3;
			}
;
false_when:	/* empty */
			{
				if(debug) std::cout << "Parsed empty(false_when)" << std::endl;
				$$ = 0;
			}
	|	FALSE_WHEN COLON bexpression
			{
				if(debug) std::cout << "Parsed FALSE_WHEN COLON bexpression" << std::endl;
				$$ = $3;
			}
;

bexpression:	bexp1
			{
				if(debug) std::cout << "Parsed bexp1" << std::endl;
				$$ = $1;
			}
	|	bexp1 AND bexpression
			{
				$1->add_boolean_expression( * $3 );
				delete $3;
				$$ = $1;
				auto a = BooleanExpression::COMP;
				$$->add_element(BooleanExpression::COMP, { .operation = BooleanExpression::AND });
			}
;
bexp1:		bexp2
			{
				if(debug) std::cout << "Parsed bexp2" << std::endl;
				$$ = $1;
			}
	|	bexp2 OR bexp1
			{
				$1->add_boolean_expression( * $3 );
				delete $3;
				$$ = $1;
				$$->add_element(BooleanExpression::COMP, { .operation = BooleanExpression::OR });
			}
;


bexp2:		OPEN_PARENTHESIS bexpression CLOSE_PARENTHESIS
			{
				$$ = $2;
			}
	|	bexp3 COMPARISON bexp3
			{
				if(debug) std::cout << "Parsed bexp3 comp bexp3" << std::endl;
				$1->add_boolean_expression( * $3 );
				delete $3;
				$$ = $1;
				$$->add_element(BooleanExpression::COMP, { .comparison = $2 });
			}
;
bexp3:		bexp4
			{
				if(debug) std::cout << "Parsed bexp4" << std::endl;
				$$ = $1;
			}
	|	bexp4 MULTIPLY bexp3
			{
				$1->add_boolean_expression( * $3 );
				delete $3;
				$$ = $1;
				$$->add_element(BooleanExpression::OP, { .operation = BooleanExpression::MUL });
			}
	|	bexp4 DIVIDE bexp3
			{
				$1->add_boolean_expression( * $3 );
				delete $3;
				$$ = $1;
				$$->add_element(BooleanExpression::OP, { .operation = BooleanExpression::DIV });
			}
;
bexp4:		bexp5
			{
				if(debug) std::cout << "Parsed bexp5" << std::endl;
				$$ = $1;
			}
	|	bexp5 PLUS bexp4
			{
				$1->add_boolean_expression( * $3 );
				delete $3;
				$$ = $1;
				$$->add_element(BooleanExpression::OP, { .operation = BooleanExpression::PLUS });
			}
	|	bexp5 MINUS bexp4
			{
				$1->add_boolean_expression( * $3 );
				delete $3;
				$$ = $1;
				$$->add_element(BooleanExpression::OP, { .operation = BooleanExpression::MINUS });
			}
;
bexp5:		INTEGER
			{
				int t = $1->get_value_int();
				$$ = new BooleanExpression();
				$$->add_element(BooleanExpression::NUMBER, { .number = t });
			}
	|	SHARP_TRUE
			{
				if(debug) std::cout << "Parsed SHARP_TRUE" << std::endl;
				$$ = new BooleanExpression();
				$$->add_element(BooleanExpression::T);
			}
	|	SHARP_FALSE
			{
				$$ = new BooleanExpression();
				$$->add_element(BooleanExpression::F);
			}
	|	SHARP_INCONSISTENT
			{
				$$ = new BooleanExpression();
				$$->add_element(BooleanExpression::I);
			}
	|	SHARP_UNKNOWN
			{
				$$ = new BooleanExpression();
				$$->add_element(BooleanExpression::U);
			}
	|	SHARP_ALL
			{
				$$ = new BooleanExpression();
				$$->add_element(BooleanExpression::ALL);
			}
;

members:	MEMBERS COLON members_list
			{
				if(debug) std::cout << "Parsed MEMBERS COLON members_list" << std::endl;
				$$ = $3;
			}
;
members_list:	ID DOT
			{
				if(debug) std::cout << "Parsed ID DOT (members_list)" << std::endl;
				$$ = new std::vector<std::string>();
				$$->push_back($1->get_value_string());
				addToDelete((void*) $$, Inter4ql::disp_type::MEMBERS_VECTOR);
			}
	|       members_list ID DOT
			{
				if(debug) std::cout << "Parsed members_list ID DOT" << std::endl;
				$$ = $1;
				$$->push_back($2->get_value_string());
			}
;
heuristics_list: NONE DOT
			{
				if(debug) std::cout << "Parsed NONE DOT" << std::endl;
				$$ = new std::vector<std::string>();
				addToDelete((void*) $$, Inter4ql::disp_type::MEMBERS_VECTOR);
				$$->push_back("none");
			}
	|	heuristics_list_internal
			{
				$$ = $1;
			}
;
heuristics_list_internal:
		heuristics
			{
				if(debug) std::cout << "Parsed heuristics (heuristics_list_internal)" << std::endl;
				$$ = new std::vector<std::string>();
				addToDelete((void*) $$, Inter4ql::disp_type::MEMBERS_VECTOR);
				$$->push_back($1->get_value_string());
				removeFromDelete((void*) $1, Inter4ql::disp_type::VALUE);
				delete $1;
			}
	|	heuristics_list_internal heuristics
			{
				if(debug) std::cout << "Parsed heuristics_list_internal heuristics" << std::endl;
				$1->push_back($2->get_value_string());
				removeFromDelete((void*) $2, Inter4ql::disp_type::VALUE);
				delete $2;
				$$ = $1;
			}
;
heuristics:	RANDOM DOT
			{
				if(debug) std::cout << "Parsed RANDOM DOT" << std::endl;
				$$ = new Value("random");
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
	|	COMPOSITE_FIRST DOT
			{
				if(debug) std::cout << "Parsed COMPOSITE_FIRST DOT" << std::endl;
				$$ = new Value("composite_first");
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
	|	GOAL_PROBING DOT
			{
				if(debug) std::cout << "Parsed GOAL_PROBING DOT" << std::endl;
				$$ = new Value("goal_probing");
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
	|	INCREASE_RATING_ONLY DOT
			{
				if(debug) std::cout << "Parsed INCREASE_RATING_ONLY DOT" << std::endl;
				$$ = new Value("increase_rating_only");
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
	|	STRICT_INCREASE_RATING_ONLY DOT
			{
				if(debug) std::cout << "Parsed STRICT_INCREASE_RATING_ONLY DOT" << std::endl;
				$$ = new Value("strict_increase_rating_only");
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
	|	DISALLOW_FAILED_PRECONDITIONS DOT
			{
				if(debug) std::cout << "Parsed DISALLOW_FAILED_PRECONDITIONS DOT" << std::endl;
				$$ = new Value("disallow_failed_preconditions");
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
;
worlds:		WORLDS COLON string_list
			{
				if(debug) std::cout << "Parsed WORLDS COLON string_list" << std::endl;
				$$ = $3;
			}
;
string_list:	ID DOT
			{
				if(debug) std::cout << "Parsed ID DOT (string_list)" << std::endl;
				$$ = new std::vector<std::string>();
				$$->push_back($1->get_value_string());
				addToDelete((void*) $$, Inter4ql::disp_type::MEMBERS_VECTOR);
			}
	|	string_list ID DOT
			{
				if(debug) std::cout << "Parsed string_list ID DOT" << std::endl;
				$$ = $1;
				$$->push_back($2->get_value_string());
			}
;
domains:	/* empty */
			{
				if(debug) std::cout << "Parsed empty(domains)" << std::endl;
				$$ = new std::vector<Domain*>();
				addToDelete((void*) $$, Inter4ql::disp_type::DOMAIN_VECTOR);
			}
	|	DOMAINS COLON domain_declarations
			{
				if(debug) std::cout << "Parsed DOMAINS COLON domain_declarations" << std::endl;
				$$ = $3;
			}
;
domain_declarations:
		/* empty */
			{
				if(debug) std::cout << "Parsed empty(domain_declarations)" << std::endl;
				$$ = new std::vector<Domain*>();
				addToDelete((void*) $$, Inter4ql::disp_type::DOMAIN_VECTOR);
			}
	|       domain_declarations domain_declaration
			{
				if(debug) std::cout << "Parsed domain_declarations domain_declaraion" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
;
domain_declaration:
		type ID DOT
			{
				if(debug) std::cout << "Parsed type ID DOT" << std::endl;
				$$ = new Domain($2->get_value_string(), $1);
				addToDelete((void*) $$, Inter4ql::disp_type::DOMAIN_suf);
			}
;
relations:      RELATIONS COLON relation_declarations
			{
				if(debug) std::cout << "Parsed RELATIONS COLON relation_declarations" << std::endl;
				$$ = $3;
			}
;
relation_declarations:
		/* empty */
			{
				if(debug) std::cout << "Parsed empty(relation_declarations)" << std::endl;
				$$ = new std::vector<Relation*>();
				addToDelete((void*) $$, Inter4ql::disp_type::RELATION_VECTOR);
			}
	|       relation_declarations relation_declaration
			{
				if(debug) std::cout << "Parsed relations_declarations relation_declaration" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
;
relation_declaration:
		ID OPEN_PARENTHESIS parameters CLOSE_PARENTHESIS DOT
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS parameters CLOSE_PARENTHESIS DOT" << std::endl;
				$$ = new Relation($1->get_value_string(), $3, std::string(""));
				addToDelete((void*) $$, Inter4ql::disp_type::RELATION);
			}
;
rules:		/* empty */
			{
				if(debug) std::cout << "Parsed empty(rules)" << std::endl;
				$$ = new std::vector<Rule *>();
				addToDelete((void*) $$, Inter4ql::disp_type::RULE_VECTOR);
			}
	|       RULES COLON rule_declarations
			{
				if(debug) std::cout << "Parsed RULES COLON rule_declarations" << std::endl;
				$$ = $3;
			}
;
rule_declarations:
		/* empty */
			{
				if(debug) std::cout << "Parsed empty(rule_declarations)" << std::endl;
				$$ = new std::vector<Rule *>();
				addToDelete((void*) $$, Inter4ql::disp_type::RULE_VECTOR);
			}
	|	rule_declarations rule_declaration
			{
				if(debug) std::cout << "Parsed rule_declarations rule_declaration" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
;
rule_declaration:
		rule_head IMPL relations_list DOT
			{
				if(debug) std::cout << "Parsed rule_head IMPL relations_list DOT" << std::endl;
				$$ = new Rule($1, $3);
				addToDelete((void*) $$, Inter4ql::disp_type::RULE);
			}

	|	rule_head DOT
			{
				if(debug) std::cout << "Parsed rule_head DOT" << std::endl;

				std::vector<std::vector<Fact*>*>* rule_body = new std::vector<std::vector<Fact*>*>();
				addToDelete((void*) rule_body, Inter4ql::disp_type::FACT_VECTOR_VECTOR);
				std::vector<Fact*> *rule_body_section = new std::vector<Fact*>();
				addToDelete((void*) rule_body_section, Inter4ql::disp_type::FACT_VECTOR);

				std::vector<Value*>* arguments = new std::vector<Value *>();
				addToDelete((void*) arguments, Inter4ql::disp_type::VALUE_VECTOR);
				Value* temp = new Value("true");
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				arguments->push_back(temp);
				Fact* rule_fact  = new Fact("math", "logic_value", arguments);
				addToDelete((void*) rule_fact, Inter4ql::disp_type::FACT);
			
				rule_body_section->push_back(rule_fact);
				rule_body->push_back(rule_body_section);

				$$ = new Rule($1, rule_body);
				addToDelete((void*) $$, Inter4ql::disp_type::RULE);
			}
;
rule_head:
		ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact(std::string(""), $1->get_value_string(), $3, true, false);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|       NOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed NOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact(std::string(""), $2->get_value_string(), $4, false, true);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
;
relation_in_rule_curly:
		relation_in_rule
			{
				if(debug) std::cout << "Parsed relation_in_rule" << std::endl;
				$$ = $1;
			}
	|       relation_in_rule IN OPEN_CURLY logic_list CLOSE_CURLY
			{
				if(debug) std::cout << "Parsed relation_in_rule IN OPEN_CURLY logic_list CLOSE_CURLY" << std::endl;
				$1->set_set($4);
				$$ = $1;
			}
	|       relation_in_rule ASSIGN oneelem_logic_list
			{
				if(debug) std::cout << "Parsed relation_in_rule IN OPEN_CURLY logic_list CLOSE_CURLY" << std::endl;
				$1->set_set($3);
				$$ = $1;
			}
;

relation_in_rule:
		ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact(std::string(""), $1->get_value_string(), $3, true, false);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|       NOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed NOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact(std::string(""), $2->get_value_string(), $4, false, true);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|       ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact($1->get_value_string(), $3->get_value_string(), $5, true, false);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|       NOT ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed NOT ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact($2->get_value_string(), $4->get_value_string(), $6, false, true);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|       OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				Fact* t = new Fact("", $5->get_value_string(), $7, true, false);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				Expression* e = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				addToDelete((void*) e, Inter4ql::disp_type::EXPRESSION);
				$$ = new Fact("BEL", std::string($2->get_value_string()), e, true, false);
				removeFromDelete((void*) $2, Inter4ql::disp_type::VALUE);
				delete $2;
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|       NOT OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed NOT OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				Fact* t = new Fact("", $6->get_value_string(), $8, true, false);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				Expression* e = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				addToDelete((void*) e, Inter4ql::disp_type::EXPRESSION);
				$$ = new Fact("BEL", std::string($3->get_value_string()), e, false, true);
				removeFromDelete((void*) $3, Inter4ql::disp_type::VALUE);
				delete $3;
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|	ID DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed ID DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact("BEL", $1->get_value_string(), $4, true, false);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|	NOT ID DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed NOT ID DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact("BEL", $2->get_value_string(), $5, false, true);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|	OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact("BEL", std::string($2->get_value_string()), $6, true, false);
				removeFromDelete((void*) $2, Inter4ql::disp_type::VALUE);
				delete $2;
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|	NOT OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed NOT OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact("BEL", std::string($3->get_value_string()), $7, false, true);
				removeFromDelete((void*) $3, Inter4ql::disp_type::VALUE);
				delete $3;
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|	bel
			{
				if(debug) std::cout << "Parsed bel from relation_in_rule" << std::endl;
				$$ = $1;
			}
	|	NOT bel
			{
				if(debug) std::cout << "Parsed NOT bel from relation_in_rule" << std::endl;
				$$ = $2;
				$$->set_value(Inter4ql::logic_type::FALSE);
			}
;

bel:		BEL OPEN_BRACKETS as_expression CLOSE_BRACKETS OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed BEL OPEN_BRACKETS as_expression CLOSE_BRACKETS OPEN_PARENTHESIS expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Fact("BEL", std::string($3->get_value_string()), $6, true, false);
				$$->set_bel_free_query(false);
				removeFromDelete((void*) $3, Inter4ql::disp_type::VALUE);
				delete $3;
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
;
as_expression:
		ID
			{
				if(debug) std::cout << "Parsed ID from as_expression" << std::endl;
				$$ = new Value($1->get_value_string());
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
	|	as_expression AS as_expression
			{
				if(debug) std::cout << "Parsed as_expression AS as_expression" << std::endl;
				$$ = new Value("<" + $1->get_value_string() + "," + $3->get_value_string() + ">");
				removeFromDelete((void*) $1, Inter4ql::disp_type::VALUE);
				delete $1;
				removeFromDelete((void*) $3, Inter4ql::disp_type::VALUE);
				delete $3;
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE);
			}
	|	OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS" << std::endl;
				$$ = $2;
			}
;
relations_list:
		relation_in_rule_curly
			{
				if(debug) std::cout << "Parsed reation_in_rule_curly" << std::endl;
				$$ = new std::vector<std::vector<Fact*>*>();
				addToDelete((void*) $$, Inter4ql::disp_type::FACT_VECTOR_VECTOR);
				std::vector<Fact*> *v = new std::vector<Fact*>();
				addToDelete((void*) v, Inter4ql::disp_type::FACT_VECTOR);
				v->push_back($1);
				$$->push_back(v);
			}
	|       relations_list COMMA relation_in_rule_curly
			{
				if(debug) std::cout << "Parsed relations_list COMMA relations_in_rule_curly" << std::endl;
				$1->at($1->size()-1)->push_back($3);
				$$ = $1;
			}
	|       relations_list OR relation_in_rule_curly
			{
				if(debug) std::cout << "Parsed relations_list OR relation_in_rule_curly" << std::endl;
				std::vector<Fact*> *v = new std::vector<Fact*>();
				addToDelete((void*) v, Inter4ql::disp_type::FACT_VECTOR);
				v->push_back($3);
				$1->push_back(v);
				$$ = $1;
			}
;
arguments:	argument
			{
				if(debug) std::cout << "Parsed argument" << std::endl;
				$$ = new std::vector<Value *>();
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE_VECTOR);
				Value* temp = new Value($1);
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				$$->push_back(temp);
			}
	|       arguments COMMA argument
			{
				if(debug) std::cout << "Parsed arguments COMMA argument" << std::endl;
				Value* temp = new Value($3);
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				$1->push_back(temp);
				$$ = $1;
			}
;

arguments_empty:/* empty */
			{
				if(debug) std::cout << "Parsed empty argument list" << std::endl;
				$$ = new std::vector<Value *>();
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE_VECTOR);
			}
;

argument:       ID
	|       VARIABLE
	|       STRING
	|       INTEGER
	|       REAL
	|       LOGIC
	|       DATE
	|       DATE_TIME
;

oneelem_logic_list:
		LOGIC
			{
				if(debug) std::cout << "Parsed oneelem_logic_list (LOGIC)" << std::endl;
				$$ = new std::vector<Value *>();
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE_VECTOR);
				Value* temp = new Value($1);
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				$$->push_back(temp);
			}	  
;

logic_list:
		oneelem_logic_list
			{
				if(debug) std::cout << "Parsed logic_list (oneelem_logic_list)" << std::endl;
				$$ = $1;
			}
	|       logic_list COMMA LOGIC
			{
				if(debug) std::cout << "Parsed logic_list COMMA LOGIC" << std::endl;
				Value* temp = new Value($3);
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				$1->push_back(temp);
				$$ = $1;
			}
;

facts:		/* empty */
			{
				if(debug) std::cout << "Parsed empty(facts)" << std::endl;
				$$ = new std::vector<Fact *>();
				addToDelete((void*) $$, Inter4ql::disp_type::FACT_VECTOR);
			}
	|       FACTS COLON fact_declarations
			{
				if(debug) std::cout << "Parsed FACTS COLON fact_declarations" << std::endl;
				$$ = $3;
			}
;
fact_declarations:
		/* empty */
			{
				if(debug) std::cout << "Parsed empty(fact_declarations)" << std::endl;
				$$ = new std::vector<Fact *>();
				addToDelete((void*) $$, Inter4ql::disp_type::FACT_VECTOR);
			}
	|       fact_declarations fact_declaration
			{
				if(debug) std::cout << "Parsed fact_declarations fact_declaration" << std::endl;
				$1->push_back($2);
				$$ = $1;
			}
;
fact_declaration:
		ID OPEN_PARENTHESIS values CLOSE_PARENTHESIS DOT
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS values CLOSE_PARENTHESIS DOT" << std::endl;
				$$ = new Fact(std::string(""), $1->get_value_string(), $3, true, false);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
	|       NOT ID OPEN_PARENTHESIS values CLOSE_PARENTHESIS DOT
			{
				if(debug) std::cout << "Parsed NOT ID OPEN_PARENTHESIS values CLOSE_PARENTHESIS DOT" << std::endl;
				$$ = new Fact(std::string(""), $2->get_value_string(), $4, false, true);
				addToDelete((void*) $$, Inter4ql::disp_type::FACT);
			}
;

values:		value
			{
				if(debug) std::cout << "Parsed value" << std::endl;
				$$ = new std::vector<Value *>();
				addToDelete((void*) $$, Inter4ql::disp_type::VALUE_VECTOR);
				Value* temp = new Value($1);
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				$$->push_back(temp);
			}
	|       values COMMA value
			{
				if(debug) std::cout << "Parsed values COMMA value" << std::endl;
				Value* temp = new Value($3);
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				$1->push_back(temp);
				$$ = $1;
			}
;
value:		ID
	|       STRING
	|       INTEGER
	|       REAL
	|       LOGIC
	|       DATE
	|       DATE_TIME
;

type:		CONST_T
			{
				if(debug) std::cout << "Parsed CONST_T" << std::endl;
				$$ = variable_type::ID;
			}
	|       STRING_T
			{
				if(debug) std::cout << "Parsed STRING_T" << std::endl;
				$$ = variable_type::STRING;
			}
	|       INTEGER_T
			{
				if(debug) std::cout << "Parsed INTEGER_T" << std::endl;
				$$ = variable_type::INTEGER;
			}
	|       REAL_T
			{
				if(debug) std::cout << "Parsed REAL_T" << std::endl;
				$$ = variable_type::REAL;
			}
	|       LOGIC_T
			{
				if(debug) std::cout << "Parsed LOGIC_T" << std::endl;
				$$ = variable_type::LOGIC;
			}
	|       DATE_T
			{
				if(debug) std::cout << "Parsed DATE_T" << std::endl;
				$$ = variable_type::DATE;
			}
	|       DATE_TIME_T
			{
				if(debug) std::cout << "Parsed DATE_TIME_T" << std::endl;
				$$ = variable_type::DATE_TIME;
			}
;
parameters:	ID
			{
				if(debug) std::cout << "Parsed ID" << std::endl;
				$$ = new std::vector<Domain *>();
				addToDelete((void*) $$, Inter4ql::disp_type::DOMAIN_VECTOR);
				Domain* temp = new Domain($1->get_value_string());
				addToDelete((void*) temp, Inter4ql::disp_type::DOMAIN_suf);
				$$->push_back(temp);
			}
	|       type    {
				if(debug) std::cout << "Parsed type" << std::endl;
				$$ = new std::vector<Domain *>();
				addToDelete((void*) $$, Inter4ql::disp_type::DOMAIN_VECTOR);
				Domain* temp = new Domain(variable_type2string($1),$1);
				addToDelete((void*) temp, Inter4ql::disp_type::DOMAIN_suf);
				$$->push_back(temp);
			}
	|	parameters COMMA ID
			{
				if(debug) std::cout << "Parsed parameters COMMA ID" << std::endl;
				Domain* temp = new Domain($3->get_value_string());
				addToDelete((void*) temp, Inter4ql::disp_type::DOMAIN_suf);
				$1->push_back(temp);
				$$ = $1;
			}
	|	parameters COMMA type
			{
				if(debug) std::cout << "Parsed parameters COMMS type" << std::endl;
				Domain* temp = new Domain(variable_type2string($3),$3);
				addToDelete((void*) temp, Inter4ql::disp_type::DOMAIN_suf);
				$1->push_back(temp);
				$$ = $1;
			}
;
query:		expression
			{
				if(debug) std::cout << "Parsed expression" << std::endl;
				//$3->set_module($1->get_value_string());
				$$ = $1;
			}
	|	OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT expression
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS as_expression CLOSE_PARENTHESIS DOT expression" << std::endl;
				Fact* f = new Fact("BEL", $2->get_value_string(), $5, true, false);
				removeFromDelete((void*) $2, Inter4ql::disp_type::VALUE);
				delete $2;
				addToDelete((void*) f, Inter4ql::disp_type::FACT);
				Expression* e = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, f);
				addToDelete((void*) e, Inter4ql::disp_type::EXPRESSION);
				e->set_module("BEL");
				$$ = e;
			}
;
simple_relation:
		ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS (simple_relation)" << std::endl;
				Fact* t = new Fact("", $1->get_value_string(), $3, true, false);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				$$ = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				$$->set_simple_relation_flag(true);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	ID OPEN_PARENTHESIS arguments_empty CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS arguments_empty CLOSE_PARENTHESIS (simple_relation)" << std::endl;
				Fact* t = new Fact("", $1->get_value_string(), $3, true, false);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				$$ = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				$$->set_simple_relation_flag(true);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
;
expression:	expression_entry
			{
				if(debug) std::cout << "Parsed expression_entry (expression)" << std::endl;
				$$ = $1;
			}
	|	OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
			{
				$$ = $2;
			}
;
expression_entry:
		simple_relation
			{
				if(debug) std::cout << "Parsed simple_relation (expression)" << std::endl;
				$$ = $1;
			}
	|	ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS" << std::endl;
				Fact* t = new Fact($1->get_value_string(), $3->get_value_string(), $5, true, false);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				$$ = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS IN OPEN_CURLY logic_list CLOSE_CURLY
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS IN OPEN_CURLY logic_list CLOSE_CURLY" << std::endl;
				Fact* t = new Fact("", $1->get_value_string(), $3, true, false);
				t->set_set($7);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				$$ = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS ASSIGN oneelem_logic_list
			{
				if(debug) std::cout << "Parsed ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS ASSIGN oneelem_logic_list" << std::endl;
				Fact* t = new Fact("", $1->get_value_string(), $3, true, false);
				t->set_set($6);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				$$ = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS IN OPEN_CURLY logic_list CLOSE_CURLY
			{
				if(debug) std::cout << "Parsed ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS IN OPEN_CURLY logic_list CLOSE_CURLY" << std::endl;
				Fact* t = new Fact($1->get_value_string(), $3->get_value_string(), $5, true, false);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				t->set_set($9);
				$$ = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS ASSIGN oneelem_logic_list
			{
				if(debug) std::cout << "Parsed ID DOT ID OPEN_PARENTHESIS arguments CLOSE_PARENTHESIS ASSIGN oneelem_logic_list" << std::endl;
				Fact* t = new Fact($1->get_value_string(), $3->get_value_string(), $5, true, false);
				addToDelete((void*) t, Inter4ql::disp_type::FACT);
				t->set_set($8);
				$$ = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, t);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	quantifier_list OPEN_PARENTHESIS expression CLOSE_PARENTHESIS IN OPEN_CURLY logic_list CLOSE_CURLY
			{
				if(debug) std::cout << "Parsed quantifier_list OPEN_PARENTHESIS expression CLOSE_PARENTHESIS IN OPEN_CURLY logic_list CLOSE_CURLY" << std::endl;
				ConstraintEntry* ce  = new ConstraintEntry($1, $3, $7);
				addToDelete((void*) ce, Inter4ql::disp_type::CONSTRAINT_ENTRY);
				$$ = new Expression(ce);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	quantifier_list OPEN_PARENTHESIS expression CLOSE_PARENTHESIS ASSIGN oneelem_logic_list
			{
				if(debug) std::cout << "Parsed quantifier_list OPEN_PARENTHESIS expression CLOSE_PARENTHESIS ASSIGN oneelem_logic_list" << std::endl;
				ConstraintEntry* ce  = new ConstraintEntry($1, $3, $6);
				addToDelete((void*) ce, Inter4ql::disp_type::CONSTRAINT_ENTRY);
				$$ = new Expression(ce);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	quantifier_list OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
			{
				if(debug) std::cout << "Parsed quantifier_list OPEN_PARENTHESIS expression CLOSE_PARENTHESIS" << std::endl;
				std::vector<Value*>* v = new std::vector<Value *>();
				addToDelete((void*) v, Inter4ql::disp_type::VALUE_VECTOR);
				Value* temp = new Value("true");
				addToDelete((void*) temp, Inter4ql::disp_type::VALUE);
				v->push_back(temp);
				ConstraintEntry* ce  = new ConstraintEntry($1, $3, v);
				addToDelete((void*) ce, Inter4ql::disp_type::CONSTRAINT_ENTRY);
				$$ = new Expression(ce);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	bel
			{
				if(debug) std::cout << "Parsed bel" << std::endl;
				Expression* e = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, $1);
				addToDelete((void*) e, Inter4ql::disp_type::EXPRESSION);
				e->set_module("BEL");
				$$ = e;
			}
	|	bel IN OPEN_CURLY logic_list CLOSE_CURLY
			{
				if(debug) std::cout << "Parsed bel IN OPEN_CURLY logic_list CLOSE_CURLY" << std::endl;
				$1->set_set($4);
				Expression* e = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, $1);
				addToDelete((void*) e, Inter4ql::disp_type::EXPRESSION);
				e->set_module("BEL");
				$$ = e;
			}
	|	bel ASSIGN oneelem_logic_list
			{
				if(debug) std::cout << "Parsed bel ASSIGN oneelem_logic_list" << std::endl;
				$1->set_set($3);
				Expression* e = new Expression(Inter4ql::expression_type::TERM, NULL, NULL, $1);
				addToDelete((void*) e, Inter4ql::disp_type::EXPRESSION);
				e->set_module("BEL");
				$$ = e;
			}
	|	expression COMMA expression
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS expression AND expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::AND, $1, $3);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	expression ARROW expression
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS expression ARROW expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::IMPLIES, $1, $3);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	expression AND expression
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS expression AND expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::AND, $1, $3);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	expression OR expression
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS expression OR expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::OR, $1, $3);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
	|	NOT expression %prec UNOT
			{
				if(debug) std::cout << "Parsed OPEN_PARENTHESIS NOT expression CLOSE_PARENTHESIS" << std::endl;
				$$ = new Expression(Inter4ql::expression_type::NOT, $2, NULL);
				addToDelete((void*) $$, Inter4ql::disp_type::EXPRESSION);
			}
;

NOT: EXCLAMATION | MINUS | TILDA;
COMPARISON:	EQ
		  {
			$$ = BooleanExpression::EQ;
		  }
	|	NEQ
		  {
			$$ = BooleanExpression::NEQ;
		  }
	|	GT
		  {
			$$ = BooleanExpression::GT;
		  }
	|	GTE
		  {
			$$ = BooleanExpression::GTE;
		  }
	|	LT
		  {
			$$ = BooleanExpression::LT;
		  }
	|	LTE
		  {
			$$ = BooleanExpression::LTE;
		  }
;

%%

void Inter4ql::LineParser::error(const Inter4ql::LineParser::location_type &loc, const std::string &msg){
	Inter4ql::LineParser::location_type *l = new Inter4ql::LineParser::location_type(loc);

	std::string location = location2string(*l);
	delete l;
	Disposable* temp = toDeleteAll;
	toDeleteAll = 0;

	throw(new Exception(msg +" at "+location+"\n", temp));
}

#include "Scanner.h"
	static int yylex(
			Inter4ql::LineParser::semantic_type *yylval,
			Inter4ql::LineParser::location_type *yylloc,
			Inter4ql::Scanner &scanner){
		return scanner.yylex(yylval, yylloc);
	}

