#include "ModuleAction.h"
#include "Result.h"

namespace Inter4ql {

	ModuleAction::ModuleAction(std::string _name,
			std::vector<Value*>* _args,
			Expression* _preconditions,
			std::vector<Rule*>* _add,
			std::vector<Rule*>* _remove,
			Expression* composite_expr)
	: name(_name),
	  args(_args),
	  preconditions(_preconditions),
	  add(_add),
	  remove(_remove),
	  composite_expr(composite_expr)
	{
		this->modules = nullptr;
	}

	ModuleAction::~ModuleAction() {
	}

	std::string ModuleAction::get_name() const {
		return this->name;
	}

	std::string ModuleAction::print() {
		std::string s("");
		s += "action " + this->name + "(";
		for (auto arg : *args) {
			s += arg->print(false);
			s += ", ";
		}
		if (!args->empty()) {
			s = s.substr(0, s.length() - 2);
		}
		s += "):\n";
		if (preconditions) {
			s += "preconditions:\n";
			s += "\t" + print_expression(preconditions) + "\n";
		}
		if (composite_expr) {
			s += "composite:\n";
			s += "\t" + print_expression(composite_expr) + "\n";
		}
		if (add && !add->empty()) {
			s += "add:\n";
			for (auto a : *add) {
				s += "\t" + a->print() + "\n";
			}
		}
		if (remove && !remove->empty()) {
			s += "remove:\n";
			for (auto r : *remove) {
				s += "\t" + r->print() + "\n";
			}
		}
		return s;
	}

	module_type::type ModuleAction::get_module_type() const {
		return module_type::ACTION;
	}

	void ModuleAction::reason()
	{

	}

	Result ModuleAction::ask_for_result(Fact * term) const
	{
		return Result();
	}

	bool ModuleAction::is_composite()
	{
		return this->composite_expr != nullptr;
	}

	Expression* ModuleAction::get_composite_expr() {
		return this->composite_expr;
	}
		
	void ModuleAction::set_modules(std::vector<ModuleGeneric *> *_modules) {
		this->modules = _modules;
	}

	std::vector<Value*>* ModuleAction::get_args() {
		return this->args;
	}

	Expression* ModuleAction::get_preconditions() {
		return this->preconditions;
	}

	std::vector<Rule*>* ModuleAction::get_add_rules() {
		return this->add;
	}

	std::vector<Rule*>* ModuleAction::get_remove_rules() {
		return this->remove;
	}
}
