#ifndef __MODULE_BELIEF_H__
#define __MODULE_BELIEF_H__

#include <vector>
#include <string>

#include "Functions.h"
#include "ModuleGeneric.h"
#include "Types.h"
#include "Constraints.h"

namespace Inter4ql {

	class Rule;
	class Domain;
	class Result;
	class Relation;
	class Fact;
	class BooleanExpression;
	/** @brief Implementation of ModuleGeneric for modules with reasoning
	 */
	class ModuleBelief : public ModuleGeneric {
	public:
		/** @brief constructor
		 *
		 * @param _name name of a module
		 * @param _constraints initial list of constraints
		 * @param _members list of members
		 */
		ModuleBelief(std::string _name,
			Constraints* _constraints,
			std::vector<std::string>* _members);
		/** @brief destructor
		 *
		 */
		~ModuleBelief();

		Relation *get_relation(std::string s) const;
		virtual std::vector<Domain*> *get_domains() const;
		std::vector<std::string> *get_members() const;
		std::string get_name() const;
		std::string print();
		module_type::type get_module_type() const;
		variable_type::type get_domain_type(std::string name);
		void add_domain(Domain *d);
		void add_relation(Relation *r);
		void reason();
		Result ask_for_result(Fact * term) const;
		Result evaluate_expression_bf(Expression * expr) const override;
		Result evaluate_expression(Expression * expr) const override;

		/** @brief setter for modules needed in this module
		 *
		 * @param _modules list of modules
		 */
		void set_modules(std::vector<ModuleGeneric *> *_modules) override;
		/** @brief getter for constraints
		 *
		 * @return list of constraints
		 */
		Constraints* get_constraints() const override { return constraints; };
		/** @brief allows for disabling constraints check
		 *
		 * @param value value to be set
		 */
		void set_disable_constraints(bool value) const override { disable_constraints = value; };
		/** @brief evaluates constraints locally, in current module
		 *
		 * @param check_rigid whether rigid constrains should be checked
		 * @param check_flexible whether flexible constraints should be checked
		 * @return whether constraints are valid
		 */
		bool check_constraints_local(bool check_rigid, bool check_flexible) const override;
		/** @brief gets a list of member module objects
		 *
		 * @return list of modules
		 */
		std::vector<ModuleGeneric*>* get_member_modules();
		/** @brief checks whether constraints are violated
		 *
		 * @return true if constraints are vioilated, false otherwise
		 */
		bool constraints_violated() override;
	private:
		bool check_single_valuation(std::vector<QuantifierEntry*>* quantifiers, int index, Result* r,
				std::map<std::string, Value*>* valuation,  std::vector<Inter4ql::Value*>* expected) const;

		std::vector<std::string> *members;
		std::vector<ModuleGeneric *> *member_modules;
		std::string name;
		Constraints* constraints;
		std::vector<Domain*> *domains;
		std::vector<ModuleGeneric *> *modules;
		std::vector<Fact*> *facts;

		mutable bool disable_constraints;
	};
}

#endif  /* __MODULE_BELIEF_H__ */

