#include <sstream>

#include "ModuleExternal.h"

#include "modules/Plugin.h"
#include "VariableSpace.h"
#include "Relation.h"
#include "Exception.h"
#include "Rule.h"

namespace Inter4ql {

	ModuleExternal::ModuleExternal(VariableSpace *_v) {
		this->variable_space = _v;
	}

	ModuleExternal::~ModuleExternal() {
		for(std::vector<Relation*>::iterator it = this->relations->begin(); it != this->relations->end(); it++) {
			delete (*it);
		}
		delete this->relations;
	}

	module_type::type ModuleExternal::get_module_type() const {
		return module_type::EXTERNAL;
	}

	Relation *ModuleExternal::get_relation(std::string s) const {
		for (int i = 0; i < this->relations->size(); i++)
			if (this->relations->at(i)->get_name() == s)
				return this->relations->at(i);
		throw new Exception(std::string("There is no relation ") + s + " in module " + this->get_name() + "!\n");
	}

	std::vector<Relation*> *ModuleExternal::get_relations() const {
		return this->relations;
	}

	std::string ModuleExternal::get_name() const {
		return this->name;
	}

	std::string ModuleExternal::print() {
		std::stringstream ss;
		ss << "module " << this->name << ":\n";
		if (this->relations->size())
			ss << "relations:\n";
		for (int i = 0; i < this->relations->size(); i++)
			ss << "\t" << this->relations->at(i)->print() << "\n";
		ss << "end.\n";
		return ss.str();
	}

}
