#ifndef __MODULEEXTERNAL_H__
#define __MODULEEXTERNAL_H__

#include <vector>
#include <string>

#include "ModuleGeneric.h"
#include "Types.h"
#include "Fact.h"

namespace Inter4ql {

	class Rule;
	class Domain;
	class Relation;
	class VariableSpace;

	/** @brief class that handles modules with support of external applications
	 */
	class ModuleExternal : public ModuleGeneric {
	public:
		/** @brief constructor
		 *
		 * @param _v pointer to VariableSpace class
		 * @param _sql pointer to SQLWrapper class
		 */
		explicit ModuleExternal(VariableSpace *_v);
		/** @brief destructor
		 *
		 */
		~ModuleExternal();
		Relation *get_relation(std::string s) const;
		std::vector<Relation*> *get_relations() const;
		std::string get_name() const;
		std::string print();
		module_type::type get_module_type() const;
		/** @brief getter for sql variable space pointer
		 *
		 * @returns VariableSpace instance pointer
		 */
		VariableSpace *get_variable_space() const;
		/** @brief VariableSpace pointer for handling all values in enviroment
		 */
		VariableSpace *variable_space;
		/** @brief list of relations inside external module */
		std::vector<Relation*> *relations;
		/** @brief name of external module */
		std::string name;
	};
}

#endif  /* __MODULEEXTERNAL_H__ */

