#ifndef __MODULEINTERNAL_H__
#define __MODULEINTERNAL_H__

#include <vector>
#include <string>

#include "ModuleGeneric.h"
#include "Types.h"
#include "Fact.h"

namespace Inter4ql {

	class Rule;
	class Domain;
	class Relation;
	class VariableSpace;

	/** @brief class that handles hardcoded relations in modules (f.e. math functions)
	 */
	class ModuleInternal : public ModuleGeneric {
	public:
		/** @brief constructor
		 *
		 * @param _v pointer to variable space
		 * @param _sql pointer to SQLWrapper implementation
		 */
		explicit ModuleInternal(VariableSpace *_v);
		 /** @brief destructor
		 */
		~ModuleInternal();
		Relation *get_relation(std::string s) const;
		std::vector<Relation*> *get_relations() const;
		virtual void generate_facts() {}
		std::string get_name() const;
		std::string print();
		module_type::type get_module_type() const;
		/** @brief getter for sql variable space pointer
		 *
		 * @returns VariableSpace instance pointer
		 */
		VariableSpace *get_variable_space() const;
		/** @brief getter for reasoning wrapper pointer
		 *
		 * @returns SQLWrapper instance pointer
		 */
		/** @brief VariableSpace pointer for handling all values in enviroment */
		VariableSpace *variable_space;
		/** @brief list of relations inside external module */
		std::vector<Relation*> *relations;
		/** @brief name of external module */
		std::string name;

	};
}

#endif /* __MODULEINTERNAL\_H__ */

