#include "ModuleSum.h"

#include "Valuation.h"

namespace Inter4ql {

	ModuleSum::ModuleSum()
	: ModuleLocal("", new Constraints(nullptr, nullptr),
			new std::vector<Domain*>(),
			new std::vector<Relation*>(),
			new std::vector<Rule*>(),
			new std::vector<Fact*>())
	{
	}

	ModuleSum::~ModuleSum() {
	}

	void add_to_database(Inter4ql::database* what, Inter4ql::database* to_what)
	{
		for(const auto& elem : *what) {
			std::string key = elem.first;
			std::shared_ptr<Valuation> val = elem.second;

			auto slot = to_what->find(key);
			if (slot == to_what->end()) {
				to_what->insert(std::make_pair(key, std::make_shared<Valuation>()));
				slot = to_what->find(key);
			}

			for(auto val_elem : *val) {
				slot->second->add_element(val_elem.first, val_elem.second);
			}
		}
	}

	void ModuleSum::add_module(ModuleGeneric* m) {
		//printf("Adding module %s\n\n", m->print().c_str());

		if (m->constraints_violated())
			return;

		Inter4ql::database* other_db = m->get_wsm_database();
		if (other_db != nullptr) {
			add_to_database(other_db, &database_);
		}

		Inter4ql::database* other_db_to_add = m->get_add_database();
		if (other_db_to_add != nullptr) {
			add_to_database(other_db_to_add, database_to_add);
		}

		Inter4ql::database* other_db_to_remove = m->get_remove_database();
		if (other_db_to_remove != nullptr) {
			add_to_database(other_db_to_remove, database_to_remove);
		}

		std::vector<Relation*> *other_relations = m->get_relations();
		if(other_relations) {
			for(auto rel: *other_relations) {
				if(!this->contains_relation(rel)) {
					std::vector<Domain *>* new_params = new std::vector<Domain *>();
					for(auto param: *rel->get_params()) {
						new_params->push_back(new Domain(param->get_name(), param->get_type()));
					}
					this->add_relation(new Relation(rel->get_name(), new_params, rel->get_module()));
				}
			}
		}

		std::vector<Domain*> *other_domains = m->get_domains();
		if(other_domains) {
			for(auto dom: *other_domains) {
				if(!this->contains_domain(dom))
					this->add_domain(new Domain(dom->get_name(), dom->get_type()));
			}
		}
	}

}
