#include <sstream>

#include "Program.h"
#include "Fact.h"
#include "Relation.h"
#include "Domain.h"
#include "Rule.h"
#include "Disposable.h"
#include "ModuleLocal.h"
#include "ModuleGeneric.h"

namespace Inter4ql {

	void add_module_to_internal_facts(Expression* e, std::string module) {
		switch(e->get_type()) {
			case expression_type::AND:
			case expression_type::OR:
			case expression_type::IMPLIES:
				add_module_to_internal_facts(e->get_left_expression(), module);
				add_module_to_internal_facts(e->get_right_expression(), module);
				break;
			case expression_type::NOT:
				add_module_to_internal_facts(e->get_left_expression(), module);
				break;
			case expression_type::QUANTIFIED:
				add_module_to_internal_facts(e->get_quantified_entry()->get_body(), module);
				break;
			case expression_type::TERM:
				Fact* term = e->get_term();
				if (term->get_module() == std::string("")) {
					//printf("Setting module %s to fact %s\n", module.c_str(), term->print().c_str());
					term->set_module(module);
				}
				break;
		}
	}

	Program::Program(std::vector<ModuleGeneric *> *_modules) {
		this->modules = _modules;
		this->modules_order = 0;
		/* postprocessing, we need to add values (module names, domain types) we did not add during parsing */
		for (int i = 0; i < _modules->size(); i++) {
			ModuleGeneric *mg = static_cast<ModuleLocal*>(_modules->at(i));
			Constraints* constr = mg->get_constraints();
			if (constr) {
				std::vector<Expression*>* hard = constr->get_hard_constraints();
				if (hard) {
					for (int j = 0; j < hard->size(); j++) {
						add_module_to_internal_facts(hard->at(j), mg->get_name());
					}
				}
				std::vector<Expression*>* soft = constr->get_soft_constraints();
				if (soft) {
					for (int j = 0; j < soft->size(); j++) {
						add_module_to_internal_facts(soft->at(j), mg->get_name());
					}
				}
			}
			if (_modules->at(i)->get_module_type() != module_type::LOCAL ) continue;
			ModuleLocal *m = static_cast<ModuleLocal*>(_modules->at(i));

			for (int j = 0; j < m->get_relations()->size(); j++) {
				Relation *r = m->get_relations()->at(j);
				for (int k = 0; k < r->get_params()->size(); k++) {
					Domain *d = r->get_params()->at(k);
					d->set_type(m->get_domain_type(d->get_name()));
				}
				r->set_module(m->get_name());
			}
			for (int j = 0; j < m->get_rules()->size(); j++) {
				Rule *r = m->get_rules()->at(j);
				r->get_head()->set_module(m->get_name());
				for (int k = 0; k < r->get_body()->size(); k++)
					for (int l = 0; l < r->get_body()->at(k)->size(); l++) {
						Fact* f = r->get_body()->at(k)->at(l);
						if (f->get_module() == std::string(""))
							f->set_module(m->get_name());
					}
			}
			for(int j = 0; j < m->get_facts()->size(); j++){
				m->get_facts()->at(j)->set_module(m->get_name());
			}
		}
	}

	Program::~Program() {
		if(!Disposable::errorMode) {
			// do nothing!
		} else {
			//std::cout << "DELETE PROGRAM*" << std::endl;
		}
	}

	std::string Program::print() {
		std::stringstream ss;
		for (int i = 0; i < this->modules->size(); i++)
			ss << this->modules->at(i)->print() << "\n";
		return ss.str();
	}

	std::vector<ModuleGeneric *> *Program::get_modules() {
		return this->modules;
	}

	void Program::set_order(std::vector<ModuleGeneric *> *_order){
		this->modules_order = _order;
		return;
	}

	std::vector<ModuleGeneric *> *Program::get_order(){
		return this->modules_order;
	}
}
