#ifndef __PROGRAM_H__
#define __PROGRAM_H__

#include <string>
#include <vector>

namespace Inter4ql {

	class ModuleGeneric;

	/** @brief class that handles 4QL programs
	 */
	class Program {
	public:
		/** @brief constructor
		 *
		 * @param _modules vector of ModuleGeneric pointers
		 */
		explicit Program(std::vector<ModuleGeneric *> *_modules);
		/** @brief destructor
		 *
		 */
		~Program();
		/** @brief returns string with program information
		 *
		 * @return program information packed in a std::string
		 */
		std::string print();
		/** @brief getter for local modules
		 *
		 * @return local modules vector
		 */
		std::vector<ModuleGeneric *> *get_modules();
		/** @brief setter for reasoning order vector
		 *
		 * @param _order desired order
		 */
		void set_order(std::vector<ModuleGeneric *> *_order);
		/** @brief getter for reasoning order vector
		 *
		 * @return order of reasoning
		 */
		std::vector<ModuleGeneric *> *get_order();
	private:
		std::vector<ModuleGeneric *> *modules;
		std::vector<ModuleGeneric *> *modules_order;

	};
}

#endif  /* __PROGRAM_H__ */

