#include "Exception.h"
#include "Result.h"
#include "Logic.h"

#include <cstdio>

namespace Inter4ql {

	class Value;

	Value Result::dummy_plug = Value(logic_type::UNKNOWN, variable_type::LOGIC);

	Result::Result() {
	}

	Result::Result(std::vector<std::string> names)
		: variable_names (names) {
	}

	Result::Result(std::vector<std::string> names, Valuation valuation)
		: variable_names (names)
		, valuation (valuation) {
	}

	Result::~Result() {
	}

	bool Result::add_element(const std::vector<Value*>& vect, logic_type::type value) {
		return valuation.add_element(vect, value);
	}

	const std::vector<std::string>& Result::get_variable_names() const {
		return variable_names;
	}

	const Valuation& Result::get_valuation() const {
		return valuation;
	}

	Valuation& Result::get_valuation_destructive() {
		return valuation;
	}

	void Result::add_variable_name(const std::string name) {
		variable_names.push_back(name);
	}

	void Result::apply_operation(operation_type::type operation, logic_type::type value) {
		auto& val1 = this->get_valuation_destructive();

		for (auto& valuation1 : val1)
		{
			switch(operation) {
			case operation_type::AND:
				valuation1.second = Logic::AND(value, valuation1.second);
				break;
			case operation_type::OR:
				valuation1.second = Logic::OR(value, valuation1.second);
				break;
			case operation_type::IMPLICATION:
				valuation1.second = Logic::IMPLICATION(value, valuation1.second);
				break;
			default:
				throw new Exception("Unsupported operation type!");
			}
		}
	}

	void Result::sum_Results_OP(Result& other, operation_type::type operation) {
		const auto names1 = this->get_variable_names();
		const auto names2 = other.get_variable_names();

		int mask1_size = names1.size();
		int mask2_size = names2.size();

//		if (mask1_size == 0 && mask2_size == 0) {
//			logic_type::type logic_value;
//			std::vector<Value *> result_var;
//
//			if (operation == operation_type::AND) {
//				logic_value = Logic::AND(this->valuation.get_first_logic_value(), other.valuation.get_first_logic_value());
//			} else if (operation == operation_type::OR) {
//				logic_value = Logic::OR(this->valuation.get_first_logic_value(), other.valuation.get_first_logic_value());
//			} else if (operation == operation_type::IMPLICATION) {
//				logic_value = Logic::IMPLICATION(this->valuation.get_first_logic_value(), other.valuation.get_first_logic_value());
//			} else if (operation == operation_type::SUM) {
//				logic_value = Logic::OR_RESULT(this->valuation.get_first_logic_value(), other.valuation.get_first_logic_value());
//			}
//
//			Valuation result_valuation;
//			result_valuation.add_element(result_var, logic_value);
//
//			this->valuation = result_valuation;
//
//			return;
//		}

		std::vector<int> mask1(mask1_size, -1);
		std::vector<int> mask2(mask2_size, -1);

		for (int i = 0; i < mask1_size; i++) {
			for (int j = 0; j < mask2_size; j++) {
				if (names1[i] == names2[j]) {
					mask1[i] = j;
					mask2[j] = i;
					break;
				}
			}
		}

		std::vector<std::string> new_variable_names = names1;
		for (int i = 0; i < mask2_size; i++)
			if (mask2[i] == -1)
				new_variable_names.push_back(names2[i]);


		const auto& val1 = this->get_valuation();
		const auto& val2 = other.get_valuation();
		Valuation result_valuation;

		std::vector<int> valuation_used_1(val1.size(), 0);
		std::vector<int> valuation_used_2(val2.size(), 0);

		int index_1 = 0;
		for (const auto& valuation1 : val1) {
			int index_2 = 0;

			for (const auto& valuation2 : val2) {
				const auto var1 = valuation1.first;
				const auto var2 = valuation2.first;
				std::vector<Value *> result_var;
				bool variable_match = true;

				for (int i = 0; i < mask1_size; i++)
				{
					if (mask1[i] != -1 && *var1[i] != *var2[mask1[i]]) {
						variable_match = false;
						break;
					}
					result_var.push_back(var1[i]);
				}

				if (!variable_match) continue;

				for (int i = 0; i < mask2_size; i++)
					if (mask2[i] == -1) {
						result_var.push_back(var2[i]);
					}

				logic_type::type logic_value;
				if (operation == operation_type::AND) {
					logic_value = Logic::AND(valuation1.second, valuation2.second);
				} else if (operation == operation_type::OR) {
					logic_value = Logic::OR(valuation1.second, valuation2.second);
				} else if (operation == operation_type::IMPLICATION) {
					logic_value = Logic::IMPLICATION(valuation1.second, valuation2.second);
				} else if (operation == operation_type::SUM) {
					logic_value = Logic::OR_RESULT(valuation1.second, valuation2.second);
				}

				result_valuation.add_element(result_var, logic_value);
				valuation_used_1[index_1] = 1;
				valuation_used_2[index_2] = 1;
				index_2++;
			}
			index_1++;
		}

		index_1 = -1;
		for (const auto& valuation1 : val1)
		{
			index_1++;
			if (valuation_used_1[index_1] == 1)
				continue;

			if (val2.size() == 0) {
				const auto var1 = valuation1.first;
				std::vector<Value *> result_var;

				for (int i = 0; i < mask1_size; i++) {
					result_var.push_back(var1[i]);
				}

				for (int i = 0; i < mask2_size; i++) {
					if (mask2[i] == -1) {
						result_var.push_back(&Result::dummy_plug);
					}
				}

				logic_type::type logic_value;
				if (operation == operation_type::AND) {
					logic_value = Logic::AND(valuation1.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::IMPLICATION) {
					logic_value = Logic::IMPLICATION(valuation1.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::OR) {
					logic_value = Logic::OR(valuation1.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::SUM) {
					logic_value = Logic::OR_RESULT(valuation1.second, logic_type::UNKNOWN);
				}

				result_valuation.add_element(result_var, logic_value);

				continue;
			}

			int index_2 = -1;
			for (const auto& valuation2 : val2) {
				index_2++;
				// all valuation have to be used here
				//if (valuation_used_2[index_2] == 1)
				//	continue;

				const auto var1 = valuation1.first;
				const auto var2 = valuation2.first;
				std::vector<Value *> result_var;

				for (int i = 0; i < mask1_size; i++) {
					result_var.push_back(var1[i]);
				}

				for (int i = 0; i < mask2_size; i++) {
					if (mask2[i] == -1) {
						result_var.push_back(var2[i]);
					}
				}

				logic_type::type logic_value;
				if (operation == operation_type::AND) {
					logic_value = Logic::AND(valuation1.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::IMPLICATION) {
					logic_value = Logic::IMPLICATION(valuation1.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::OR) {
					logic_value = Logic::OR(valuation1.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::SUM) {
					logic_value = Logic::OR_RESULT(valuation1.second, logic_type::UNKNOWN);
				}

				result_valuation.add_element(result_var, logic_value);
			}
		}

		int index_2 = -1;
		for (const auto& valuation2 : val2)
		{
			index_2++;
			if (valuation_used_2[index_2] == 1)
				continue;

			if (val1.size() == 0) {
				const auto var2 = valuation2.first;
				std::vector<Value *> result_var;

				for (int i = 0; i < mask1_size; i++) {
					if (mask1[i] != -1)
						result_var.push_back(var2[mask1[i]]);
					else
						result_var.push_back(&Result::dummy_plug);
				}

				for (int i = 0; i < mask2_size; i++) {
					if (mask2[i] == -1) {
						result_var.push_back(var2[i]);
					}
				}

				logic_type::type logic_value;
				if (operation == operation_type::AND) {
					logic_value = Logic::AND(valuation2.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::OR) {
					logic_value = Logic::OR(valuation2.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::IMPLICATION) {
					logic_value = Logic::IMPLICATION(logic_type::UNKNOWN, valuation2.second);
				} else if (operation == operation_type::SUM) {
					logic_value = Logic::OR_RESULT(valuation2.second, logic_type::UNKNOWN);
				}

				result_valuation.add_element(result_var, logic_value);

				continue;
			}

			index_1 = -1;
			for (const auto& valuation1 : val1)
			{
				index_1++;

				const auto var1 = valuation1.first;
				const auto var2 = valuation2.first;
				std::vector<Value *> result_var;

				for (int i = 0; i < mask1_size; i++) {
					if (mask1[i] != -1)
						result_var.push_back(var2[mask1[i]]);
					else
						result_var.push_back(var1[i]);
				}

				for (int i = 0; i < mask2_size; i++) {
					if (mask2[i] == -1) {
						result_var.push_back(var2[i]);
					}
				}

				logic_type::type logic_value;
				if (operation == operation_type::AND) {
					logic_value = Logic::AND(valuation2.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::OR) {
					logic_value = Logic::OR(valuation2.second, logic_type::UNKNOWN);
				} else if (operation == operation_type::IMPLICATION) {
					logic_value = Logic::IMPLICATION(logic_type::UNKNOWN, valuation2.second);
				} else if (operation == operation_type::SUM) {
					logic_value = Logic::OR_RESULT(valuation2.second, logic_type::UNKNOWN);
				}

				result_valuation.add_element(result_var, logic_value);
			}
		}

		this->variable_names = new_variable_names;
		this->valuation = result_valuation;
	}

	void Result::negate() {
		for (auto& val : get_valuation_destructive())
		{
			val.second = Logic::NOT(val.second);
		}
	}

	Result* Result::trim_variables(std::vector<std::string> names)
	{
		Valuation result;

		for (auto v_it : valuation)
		{
			std::vector<Value*> val;
			for (const auto& name : names)
			{
				int index = -1;
				for (const auto& var : variable_names)
				{
					index++;
					if (var == name) {
						break;
					}
				}
				if (index != -1)
					val.push_back(v_it.first[index]);
			}

			if (val.empty())
				return new Result(variable_names, valuation);

			//result.add_element(val, v_it.second);

			auto it = result.begin();
			for (; it != result.end() ; it++)
			{
				bool match = true;
				for (int i = 0 ; i < val.size(); i++) {
					if (*val[i] != *it->first[i]) {
						match = false;
						break;
					}
				}
				if (match)
					break;
			}

			if (it != result.end()) {
				it->second = Logic::TUP(it->second, v_it.second);
			} else {
				result.add_element(val, v_it.second);
			}

		}

		//this->variable_names = names;
		//this->valuation = result;
		return new Result(names, result);
	}

	std::string Result::print(bool type, bool unknown)
	{
		std::string res = "";

		if (this->variable_names.empty())
		{
			auto logic_value = valuation.get_first_logic_value();
			res = Logic::get_logic_name(logic_value);
			res += "\n";
			return res;
		}

		for (auto val : valuation)
		{
			if (!unknown && val.second == logic_type::UNKNOWN)
				continue;
			for (int i = 0; i < variable_names.size(); i++)
			{
				if ( i != 0) res += ", ";
				res += variable_names[i] + ": ";
				res += val.first[i]->print(type);
			}
			res += " = " + Logic::get_logic_name(val.second);
			res += "\n";
		}
		return res;

	}

}
