#ifndef __RULE_H__
#define __RULE_H__

#include <map>
#include <vector>

namespace Inter4ql {

	class Fact;
	class Value;

	/** @brief class that handles rules in ModuleLocal
	 */
	class Rule {
	public:
		/** @brief constructor that creates rule
		 *
		 * @param _head head of a rule
		 * @param _body body of a rule (as vector that holds alternative conjunction vectors)
		 */
		Rule(Fact *_head, std::vector<std::vector<Fact*>*> *_body);
		/** @brief destructor of Rule object
		 *
		 */
		~Rule();
		/** @brief constructor that creates rule
		 *
		 * @param _head head of a rule
		 * @param _body body of a rule
		 */
		Rule(Fact *_head, std::vector<Fact*> *_body);
		/** @brief constructor that creates rule
		 *
		 * @param other othe Rule to be copied
		 */
		Rule(Rule* other);
		/** @brief getter for a head Fact
		 *
		 * @return pointer to head Fact
		 */
		Fact *get_head();
		/** @brief getter for a body facts
		 *
		 * @return pointer to body of rule
		 */
		std::vector<std::vector<Fact*>*> *get_body();
		/** @brief returns string with rule information
		 *
		 * @return rule information packed in a std::string
		 */
		std::string print();

		void valuate_rule(std::map<std::string, Value*> binding);
	private:
		Fact *head;
		std::vector<std::vector<Fact*>*> *body;
	};
}

#endif  /* RULE_H */

