%{
#include "Scanner.h"
#include "Functions.h"
#include "Value.h"
#include "Disposable.h"
#include <vector>

/* make sure global variable colnum value is 0 */
#define YY_USER_INIT { colnum = 0; }
/* default action when entering a token (to be sure yylloc is proper location */
#define YY_USER_ACTION { \
	yylloc->begin.line = yylineno; \
	yylloc->begin.column = colnum; \
	colnum += yyleng;              \
	yylloc->end.line = yylineno;   \
	yylloc->end.column = colnum;   \
}

#define YY_COPY_LLOC(n) { \
	n.begin.line = yylloc->begin.line; \
	n.begin.column = yylloc->begin.column; \
	n.end.line = yylloc->end.line; \
	n.end.column = yylloc->end.column \
}

std::string strbuf;
int colnum = 0;
std::vector<Inter4ql::Value*>* toDelete = 0;
Inter4ql::Disposable* toDeleteAll = 0;

%}
%x SQUOTE
%option nodefault yylineno yyclass="Scanner" noyywrap c++
ZERO		0
NONZERO		[1-9]
DIGIT 		[0-9]
GTZERO		({NONZERO}({DIGIT})*)
INTEGER		({GTZERO}|{ZERO})
DATE		([0-3][0-9]-[0-1][0-9]-[1-2][0-9]{3})
DATE_TIME	{DATE}\ [0-2][0-9]:[0-5][0-9]

%%

modules		{ return Inter4ql::LineParser::token::MODULES; }
module		{ return Inter4ql::LineParser::token::MODULE; }
group		{ return Inter4ql::LineParser::token::GROUP; }
beliefs		{ return Inter4ql::LineParser::token::BELIEFS; }
flexible	{ return Inter4ql::LineParser::token::FLEXIBLE; }
rigid		{ return Inter4ql::LineParser::token::RIGID; }
as		{ return Inter4ql::LineParser::token::AS; }
debug		{ return Inter4ql::LineParser::token::DEBUG; }
enable		{ return Inter4ql::LineParser::token::ENABLE; }
disable		{ return Inter4ql::LineParser::token::DISABLE; }
help		{ return Inter4ql::LineParser::token::HELP; }
heuristics	{ return Inter4ql::LineParser::token::HEURISTICS; }
random		{ return Inter4ql::LineParser::token::RANDOM; }
composite_first { return Inter4ql::LineParser::token::COMPOSITE_FIRST; }
increase_rating_only { return Inter4ql::LineParser::token::INCREASE_RATING_ONLY; }
strict_increase_rating_only { return Inter4ql::LineParser::token::STRICT_INCREASE_RATING_ONLY; }
disallow_failed_preconditions { return Inter4ql::LineParser::token::DISALLOW_FAILED_PRECONDITIONS; }
goal_probing	{ return Inter4ql::LineParser::token::GOAL_PROBING; }
none		{ return Inter4ql::LineParser::token::NONE; }
Bel		{ return Inter4ql::LineParser::token::BEL; }
constraints	{ return Inter4ql::LineParser::token::CONSTRAINTS; }
forall		{ return Inter4ql::LineParser::token::FORALL; }
exists		{ return Inter4ql::LineParser::token::EXISTS; }
members		{ return Inter4ql::LineParser::token::MEMBERS; }
worlds		{ return Inter4ql::LineParser::token::WORLDS; }
true_when	{ return Inter4ql::LineParser::token::TRUE_WHEN; }
false_when	{ return Inter4ql::LineParser::token::FALSE_WHEN; }
inconsitent_when { return Inter4ql::LineParser::token::INCONSISTENT_WHEN; }
domains		{ return Inter4ql::LineParser::token::DOMAINS; }
relations	{ return Inter4ql::LineParser::token::RELATIONS; }
rules		{ return Inter4ql::LineParser::token::RULES; }
facts		{ return Inter4ql::LineParser::token::FACTS; }
end		{ return Inter4ql::LineParser::token::END; }
import		{ return Inter4ql::LineParser::token::IMPORT; }
exit		{ return Inter4ql::LineParser::token::EXIT; }
quit		{ return Inter4ql::LineParser::token::EXIT; }
print		{ return Inter4ql::LineParser::token::PRINT; }
execute		{ return Inter4ql::LineParser::token::EXECUTE; }
clear		{ return Inter4ql::LineParser::token::CLEAR; }
action		{ return Inter4ql::LineParser::token::ACTION; }
preconditions	{ return Inter4ql::LineParser::token::PRECONDITIONS; }
postconditions	{ return Inter4ql::LineParser::token::POSTCONDITIONS; }
problem		{ return Inter4ql::LineParser::token::PROBLEM; }
problems	{ return Inter4ql::LineParser::token::PROBLEMS; }
actions		{ return Inter4ql::LineParser::token::ACTIONS; }
goal		{ return Inter4ql::LineParser::token::GOAL; }
max_depth	{ return Inter4ql::LineParser::token::MAX_DEPTH; }
add		{ return Inter4ql::LineParser::token::ADD; }
remove		{ return Inter4ql::LineParser::token::REMOVE; }
composite	{ return Inter4ql::LineParser::token::COMPOSITE; }
:-		{ return Inter4ql::LineParser::token::IMPL; }
:		{ return Inter4ql::LineParser::token::COLON; }
;;		{ return Inter4ql::LineParser::token::SEMICOLON; }
,		{ return Inter4ql::LineParser::token::COMMA; }
\/\/.*$		{ }
\.		{ return Inter4ql::LineParser::token::DOT; }
\(		{ return Inter4ql::LineParser::token::OPEN_PARENTHESIS; }
\)		{ return Inter4ql::LineParser::token::CLOSE_PARENTHESIS; }
\[		{ return Inter4ql::LineParser::token::OPEN_BRACKETS; }
\]		{ return Inter4ql::LineParser::token::CLOSE_BRACKETS; }
\&		{ return Inter4ql::LineParser::token::AND; }
\|		{ return Inter4ql::LineParser::token::OR; }
\|\|		{ return Inter4ql::LineParser::token::PARALLEL; }
\;		{ return Inter4ql::LineParser::token::SEQUENTIAL; }
!		{ return Inter4ql::LineParser::token::EXCLAMATION; }
-		{ return Inter4ql::LineParser::token::MINUS; }
\+		{ return Inter4ql::LineParser::token::PLUS; }
\*		{ return Inter4ql::LineParser::token::MULTIPLY; }
\/		{ return Inter4ql::LineParser::token::DIVIDE; }
~		{ return Inter4ql::LineParser::token::TILDA; }
literal		{ return Inter4ql::LineParser::token::CONST_T; }
string		{ return Inter4ql::LineParser::token::STRING_T; }
integer		{ return Inter4ql::LineParser::token::INTEGER_T; }
real		{ return Inter4ql::LineParser::token::REAL_T; }
logic		{ return Inter4ql::LineParser::token::LOGIC_T; }
date		{ return Inter4ql::LineParser::token::DATE_T; }
date_time	{ return Inter4ql::LineParser::token::DATE_TIME_T; }
in		{ return Inter4ql::LineParser::token::IN; }
\{		{ return Inter4ql::LineParser::token::OPEN_CURLY; }
\}		{ return Inter4ql::LineParser::token::CLOSE_CURLY; }
\#true		{ return Inter4ql::LineParser::token::SHARP_TRUE; }
\#false		{ return Inter4ql::LineParser::token::SHARP_FALSE; }
\#unknown	{ return Inter4ql::LineParser::token::SHARP_UNKNOWN; }
\#inconsistent	{ return Inter4ql::LineParser::token::SHARP_INCONSISTENT; }
\#all		{ return Inter4ql::LineParser::token::SHARP_ALL; }
\-\>		{ return Inter4ql::LineParser::token::ARROW; }
\=\>		{ return Inter4ql::LineParser::token::IF_ARROW; }
\=		{ return Inter4ql::LineParser::token::ASSIGN; }
\=\=		{ return Inter4ql::LineParser::token::EQ; }
\!\=		{ return Inter4ql::LineParser::token::NEQ; }
\>		{ return Inter4ql::LineParser::token::GT; }
\>\=		{ return Inter4ql::LineParser::token::GTE; }
\<		{ return Inter4ql::LineParser::token::LT; }
\<\=		{ return Inter4ql::LineParser::token::LTE; }
true {
	yylval->value = new Value(Inter4ql::logic_type::TRUE, Inter4ql::variable_type::LOGIC);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::LOGIC;
}
false {
	yylval->value = new Value(Inter4ql::logic_type::FALSE, Inter4ql::variable_type::LOGIC);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::LOGIC;
}
unknown {
	yylval->value = new Value(Inter4ql::logic_type::UNKNOWN, Inter4ql::variable_type::LOGIC);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::LOGIC;
}
inconsistent {
	yylval->value = new Value(Inter4ql::logic_type::INCONSISTENT, Inter4ql::variable_type::LOGIC);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::LOGIC;
}
incons {
	yylval->value = new Value(Inter4ql::logic_type::INCONSISTENT, Inter4ql::variable_type::LOGIC);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::LOGIC;
}

\"	{  strbuf.clear(); strbuf += "\""; BEGIN(SQUOTE); }
<SQUOTE>{
\" {
	strbuf += "\"";
	yylval->value = new Value(strbuf);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	BEGIN(0);
	return Inter4ql::LineParser::token::STRING;
}
\\\" {
	strbuf += "\\\"";
}
[^"] {
	strbuf += yytext;
}
}
[a-z][a-zA-Z0-9_\-]* {
	yylval->value = new Value(yytext);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::ID;
}
_+[a-zA-Z0-9][a-zA-Z0-9_\-]* {
	yylval->value = new Value(yytext);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::ID;
}
[A-Z][a-zA-Z0-9_\-]* {
	yylval->value = new Value(yytext);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::VARIABLE;
}
-?{INTEGER}\.{DIGIT}* {
	yylval->value = new Value(atof(yytext));
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::REAL;
}
{DATE} {
	yylval->value = new Value(yytext);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::DATE;
}
{DATE_TIME} {
	yylval->value = new Value(yytext);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::DATE_TIME;
}
-?{INTEGER} {
	yylval->value = new Value(atoi(yytext), Inter4ql::variable_type::INTEGER);
	if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
	toDelete->push_back(yylval->value);

	//YY_COPY_LLOC(yylval->value->location);
	return Inter4ql::LineParser::token::INTEGER;
}
. {
	if(yytext[0] != ' ' && yytext[0] != '\t')
	{
		yylval->value = new Value(yytext);
		if(toDelete == 0) toDelete = new std::vector<Inter4ql::Value*>();
		toDelete->push_back(yylval->value);

		//YY_COPY_LLOC(yylval->value->location);
		return Inter4ql::LineParser::token::INVALID;
	}
}
\n { colnum = 0; }
\r\n { colnum = 0; }

%%
