#ifndef __TYPES_H__
#define __TYPES_H__

#include <vector>
#include <string>
#include <unordered_map>
#include <memory>
#include <map>

namespace Inter4ql {

	class Program;
	class Fact;
	class Expression;
	class Value;
	class Valuation;

	typedef std::unordered_multimap<std::string, std::shared_ptr<Valuation> > database;
	//typedef std::multimap<std::string, std::shared_ptr<Valuation> > database;

	/** @brief structure that handles enumeration type for types of values.
	 */
	struct variable_type {
		/** @brief Enumeration type for types of values.
		 */
		enum type {
			TYPE = 1,
			VARIABLE = 2,
			ID = 3,
			STRING = 4,
			INTEGER = 5,
			REAL = 6,
			DATE = 7,
			DATE_TIME = 8,
			LOGIC = 9,
			INVALID = 10
		};
	};

	/** @brief structure that handles enumeration type for four-valued logic.
	 */
	struct logic_type {
		/** @brief Enumeration type for four-valued logic.
		 */
		enum type {
			TRUE = 1,
			FALSE = 2,
			UNKNOWN = 3,
			INCONSISTENT = 4
		};
	};

	/** @brief structure that handles enumeration type for command line types
	 */
	struct line_type {
		/** @brief Enumeration type for command line types
		 */
		enum type {
			PROGRAM,
			PRINT,
			IMPORT,
			DEBUG,
			HELP,
			RULE,
			QUIT,
			MODULES,
			ACTIONS,
			PROBLEMS,
			CLEAR,
			EXECUTE
		};
	};

	/** @brief structure for handling lines from interpreter
	 */
	struct parser_line {
		/** @brief type of a line
		 */
		line_type::type type;
		/** @brief value of a line (used only in some cases)
		 */
		std::string value;
		/** @brief bool value of a line (used only in some cases)
		 */
		bool bool_value;
		/** @brief values of a line (used only in some cases)
		 */
		std::vector<std::string> values;
		/** @brief pointer to a Fact in a line (used only in some cases)
		 */
		Fact *rule;
		/** @brief pointer to an Expression in a line (used only in some cases)
		 */
		Expression *expression;
		/** @brief pointer to a Program imported in a line (used only in some cases)
		 */
		Program *program;
	};

	struct module_type {
		enum type {
			LOCAL,
			INTERNAL,
			EXTERNAL,
			GROUP,
			BELIEF,
			ACTION,
			PROBLEM,
			BEL
		};
	};

	struct operation_type {
		enum type {
			AND,
			OR,
			SUM,
			IMPLICATION
		};
	};

	struct expression_type {
		enum type {
			AND,
			OR,
			IMPLIES,
			NOT,
			QUANTIFIED,
			TERM,
			PARALLEL,
			SEQUENTIAL,

		};
	};
}

#endif  /* __TYPES_H__ */

