#ifndef __VALUATION_CC__
#define __VALUATION_CC__

#include "Valuation.h"

#include <utility>
#include <iostream>

namespace Inter4ql {

	Valuation::Valuation()
	{

	}

	bool Valuation::add_element(const std::vector<Value*>& vect, logic_type::type value) {
		auto it = valuation.begin();
		for (; it != valuation.end() ; it++)
		{
			bool match = true;
			for (int i = 0 ; i < vect.size(); i++) {
				if (*vect[i] != *it->first[i]) {
					match = false;
					break;
				}
			}
			if (match)
				break;
		}

		bool new_element_added = true;

		if (it != valuation.end()) {
			auto old_value = it->second;
			it->second = Logic::OR_RESULT(it->second, value);
			new_element_added = Logic::is_more_knowledge(it->second, old_value);
		} else {
			valuation.insert(make_pair(vect, value));
		}
		return new_element_added;
	}

	logic_type::type Valuation::get_logic_value(const std::vector<Value*>& vect) {
		auto it = valuation.begin();
		for (; it != valuation.end() ; it++)
		{
			bool match = true;
			for (int i = 0 ; i < vect.size(); i++) {
				if (*vect[i] != *it->first[i]) {
					match = false;
					break;
				}
			}
			if (match)
				break;
		}

		if (it == valuation.end())
			return logic_type::UNKNOWN;

		return it->second;
	}

	bool Valuation::remove_element(const std::vector<Value*>& vect) {
		auto it = valuation.begin();
		for (; it != valuation.end() ; it++)
		{
			bool match = true;
			for (int i = 0 ; i < vect.size(); i++) {
				if (*vect[i] != *it->first[i]) {
					match = false;
					break;
				}
			}
			if (match)
				break;
		}

		if (it != valuation.end()) {
			valuation.erase(it);
			return true;
		}
		return false;
	}

	void Valuation::expand_by(const std::vector<Value *>& vals)
	{
		std::map<std::vector<Value *>, logic_type::type> new_valuation;
		for (const auto& elem : valuation)
		{
			for (const auto& val : vals)
			{
				auto new_val = elem.first;
				new_val.push_back(val);
				new_valuation.insert(make_pair(new_val, elem.second));
			}
		}
		this->valuation = new_valuation;

	}

	void Valuation::print()
	{
		for (const auto & elem : valuation)
		{
			switch(elem.second)
			{
			case logic_type::INCONSISTENT:
				std::cout << "INCONSISTENT (";
				break;
			case logic_type::TRUE:
				std::cout << "TRUE (";
				break;
			case logic_type::FALSE:
				std::cout << "FALSE (";
				break;
			case logic_type::UNKNOWN:
				std::cout << "UNKNOWN (";
				break;
			}

			for (const auto & value : elem.first)
			{
				std::cout << value->print(true) << ";";
			}

			std::cout << ")" << std::endl;
		}
	}

	bool Valuation::is_all_unknown() const {
		for (const auto& elem : valuation)
		{
			if (elem.second != logic_type::UNKNOWN)
				return false;
		}
		return true;
	}

 	logic_type::type Valuation::get_first_logic_value() const {
		for (const auto& elem : valuation)
		{
			return elem.second;
		}
		return logic_type::UNKNOWN;
	}

	void Valuation::remove_inconsistent(const Valuation& val)
	{
		/*this->valuation.erase(
			std::remove_if(
				this->valuation.begin(),
				this->valuation.end(),
				[val, this] (const std::pair<std::vector<Value*>, logic_type::type> element) {
					const std::vector<Value *> v = element.first;
					return (val.find(v) != val.end());
				}
			),
			this->valuation.end()
		);*/

		// Below solution is O(n^2), but I was not able to get above one to work ( O(n log n) )
		for (const auto & elem : val)
		{
			if (elem.second == logic_type::INCONSISTENT)
				this->valuation.erase(elem.first);
		}
	}

}
#endif
