#include <sstream>

#include "VariableSpace.h"
#include "Value.h"
#include "Exception.h"

namespace Inter4ql {

	using std::string;
	void VariableSpace::initialize(string module) {
#define MAP_INITIALIZATION(name, type) \
	this->name ## _map->insert ( std::pair<string, std::map<type, Value*>*> ( module + "." + #name, new std::map<type, Value*> ( )));

		MAP_INITIALIZATION(integer, int)
		MAP_INITIALIZATION(real, double)
		MAP_INITIALIZATION(string, string)
		MAP_INITIALIZATION(const, string)
		MAP_INITIALIZATION(date, string)
		MAP_INITIALIZATION(datetime, string)
		MAP_INITIALIZATION(logic, int)

		std::map<string, std::map<int, Value *>*>::iterator logicIt = this->logic_map->find(module+".logic");
		if(logicIt == this->logic_map->end()) {
				throw new Exception(string("No 'logic' domain created, cannot continue...\n"));
		}

		logicIt->second->clear();

		Value *vf = new Value(Inter4ql::logic_type::FALSE, Inter4ql::variable_type::LOGIC);
		Value *vt = new Value(Inter4ql::logic_type::TRUE, Inter4ql::variable_type::LOGIC);
		Value *vi = new Value(Inter4ql::logic_type::INCONSISTENT, Inter4ql::variable_type::LOGIC);
		Value *vu = new Value(Inter4ql::logic_type::UNKNOWN, Inter4ql::variable_type::LOGIC);
		logicIt->second->insert(std::pair<int, Value *>(Inter4ql::logic_type::FALSE, vf));
		logicIt->second->insert(std::pair<int, Value *>(Inter4ql::logic_type::TRUE, vt));
		logicIt->second->insert(std::pair<int, Value *>(Inter4ql::logic_type::INCONSISTENT, vi));
		logicIt->second->insert(std::pair<int, Value *>(Inter4ql::logic_type::UNKNOWN, vu));
	}

	VariableSpace::VariableSpace() {
#define MAP_CREATION(name, type) \
		this->name ## _map = new std::map<string, std::map<type, Value *>*>();

		MAP_CREATION(integer, int)
		MAP_CREATION(real, double)
		MAP_CREATION(string, string)
		MAP_CREATION(const, string)
		MAP_CREATION(date, string)
		MAP_CREATION(datetime, string)
		MAP_CREATION(logic, int)

		initialize("math");
		initialize("plugin");
	}

	VariableSpace::~VariableSpace() {
#define MAP_CLEAR(name, type) \
	for(std::map<string, std::map<type, Value*>*>::iterator it = this->name ## _map->begin(); it != this->name ## _map->end(); it++) { \
		for(std::map<type, Value*>::iterator it1 = it->second->begin(); it1 != it->second->end(); it1++) { \
			delete it1->second; \
		} \
		it->second->clear(); \
		delete it->second; \
	} \
	this->name ## _map->clear();

#define MAP_DESTROY(name, type) \
	MAP_CLEAR(name, type) \
	delete name ## _map;

		MAP_DESTROY(integer, int)
		MAP_DESTROY(real, double)
		MAP_DESTROY(string, string)
		MAP_DESTROY(const, string)
		MAP_DESTROY(date, string)
		MAP_DESTROY(datetime, string)
		MAP_DESTROY(logic, int)
	}

	void VariableSpace::clear(){
		MAP_CLEAR(integer, int)
		MAP_CLEAR(real, double)
		MAP_CLEAR(string, string)
		MAP_CLEAR(const, string)
		MAP_CLEAR(date, string)
		MAP_CLEAR(datetime, string)
		MAP_CLEAR(logic, int)

		initialize("math");
		initialize("plugin");
		return;
	}

	Value *VariableSpace::get_pointer(Value *_v, string _module, string _domain, bool& _changed) {
		if (_v->get_type() == Inter4ql::variable_type::INTEGER)
			return this->get_integer(_v, _module, _domain, _changed);
		if (_v->get_type() == Inter4ql::variable_type::REAL)
			return this->get_real(_v, _module, _domain, _changed);
		if (_v->get_type() == Inter4ql::variable_type::STRING)
			return this->get_string(_v, _module, _domain, _changed);
		if (_v->get_type() == Inter4ql::variable_type::ID)
			return this->get_const(_v, _module, _domain, _changed);
		if (_v->get_type() == Inter4ql::variable_type::DATE)
			return this->get_date(_v, _module, _domain, _changed);
		if (_v->get_type() == Inter4ql::variable_type::DATE_TIME)
			return this->get_datetime(_v, _module, _domain, _changed);
		if (_v->get_type() == Inter4ql::variable_type::LOGIC)
			return this->get_logic(_v, _module, _domain, _changed);
		if (_v->get_type() == Inter4ql::variable_type::VARIABLE)
			throw new Exception(string("Trying to put VARIABLE type into values space!\n"));

		//std::cout << "type: " << variable_type2string(_v.get_type()) << "\n";
		throw new Exception(string("Unrecognized variable put in variable space!\n"));
	}

	string VariableSpace::print() {
		std::stringstream ss;
		ss << "variable space: ";

#define MAP_PRINT(name, type) \
		ss << "\n\t" #name "s"; \
		for (std::map<string, std::map<type, Value*>*>::iterator i = this->name ## _map->begin(); i != name ## _map->end(); i++) { \
			ss << "\n\t\t" << i->first << ": "; \
			for (std::map<type, Value*>::iterator i1 = i->second->begin(); i1 != i->second->end(); i1++) { \
						ss << i1->second->print(false) << " "; \
			} \
		}

		MAP_PRINT(integer, int)
		MAP_PRINT(real, double)
		MAP_PRINT(string, string)
		MAP_PRINT(const, string)
		MAP_PRINT(date, string)
		MAP_PRINT(datetime, string)
		MAP_PRINT(logic, int)

		ss << "\n";

		//for(std::map<string, Value*>::iterator i = this->variable_map.begin(); i != variable_map.end(); i++){
		//    ss << i->second->print(false) << " ";
		//}
		//ss << "\n";

		return ss.str();
	}

#define GET_SPACE(name, type) \
	std::map<type, Value *> *VariableSpace::get_ ## name ## _space(string _module, string _domain) { \
		std::map<string, std::map<type, Value*>*>::iterator it = this->name ## _map->find(_module+"."+_domain); \
		if(it == this->name ## _map->end()) { \
			this->name ## _map->insert(std::pair<string, std::map<type, Value*>*>(_module+"."+_domain, new std::map<type, Value*>())); \
			it = this->name ## _map->find(_module+"."+_domain); \
		} \
		return it->second; \
	} \

	GET_SPACE(integer, int)
	GET_SPACE(real, double)
	GET_SPACE(string, string)
	GET_SPACE(const, string)
	GET_SPACE(date, string)
	GET_SPACE(datetime, string)
	GET_SPACE(logic, int)

#define GET_VALUE(name, type) \
	Value *VariableSpace::get_ ## name(Value *_v, string _module, string _domain, bool& _changed) { \
		std::map<string, std::map<type, Value*>*>::iterator dom = this->name ## _map->find(_module+"."+_domain); \
		if(dom == this->name ## _map->end()) { \
				this->name ## _map->insert(std::pair<string, std::map<type, Value*>*>(_module+"."+_domain, new std::map<type, Value*>())); \
				dom = this->name ## _map->find(_module+"."+_domain); \
		} \
		std::map<type, Value*>::iterator it = dom->second->find(_v->get_value_ ## type()); \
		if (it == dom->second->end()) { \
			Value *new_value = new Value(*_v); \
			dom->second->insert(std::pair<type, Value*>(_v->get_value_ ## type(), new_value)); \
			_changed |= true; \
			return new_value; \
		} \
		_changed |= false; \
		return it->second; \
	} \

	GET_VALUE(integer, int)
	GET_VALUE(real, double)
	GET_VALUE(string, string)
	GET_VALUE(const, string)
	GET_VALUE(date, string)
	GET_VALUE(datetime, string)
	GET_VALUE(logic, int)

}
