#include "../Domain.h"
#include "../VariableSpace.h"
#include "../Functions.h"
#include "Math.h"
#include <math.h>
#include <algorithm>
#include "../Exception.h"
#include "../Result.h"
#include <dirent.h>
#include <string.h>
#include <fstream>
#include <sstream>
#include <string>
#include <iostream>

namespace Inter4ql {
    Math::Math(VariableSpace *_vs) : ModuleInternal(_vs) {
        this->name = "math";
        this->relations = new std::vector<Relation *>();

        this->add_relation("lt", "INTEGER INTEGER");
        this->add_relation("ltR", "REAL REAL");

        this->add_relation("le", "INTEGER INTEGER");
        this->add_relation("leR", "REAL REAL");

        this->add_relation("gt", "INTEGER INTEGER");
        this->add_relation("gtR", "REAL REAL");

        this->add_relation("ge", "INTEGER INTEGER");
        this->add_relation("geR", "REAL REAL");

        this->add_relation("eq", "LITERAL LITERAL");
        this->add_relation("eqR", "REAL REAL");
        this->add_relation("eqI", "INTEGER INTEGER");
        this->add_relation("eqS", "STRING STRING");
        this->add_relation("eqD", "DATE DATE");
        this->add_relation("eqDT", "DATE_TIME DATE_TIME");
        this->add_relation("eqL", "LOGIC LOGIC");

        // X Y Z - if X + Y = Z then TRUE else FALSE
        this->add_relation("sum", "INTEGER INTEGER INTEGER");
        this->add_relation("sumR", "REAL REAL REAL");

        // X Y Z - if X * Y = Z then TRUE else FALSE
        this->add_relation("product", "INTEGER INTEGER INTEGER");
        this->add_relation("productR", "REAL REAL REAL");

        // X Y Z - if X * Y = Z then TRUE else FALSE
        this->add_relation("quotient", "INTEGER INTEGER REAL");
        this->add_relation("quotientR", "REAL REAL REAL");

        // X Y Z - if Z = min{X, Y} then TRUE else FALSE
        this->add_relation("min", "INTEGER INTEGER INTEGER");
        this->add_relation("minR", "REAL REAL REAL");

        // X Y Z - if Z = max{X, Y} then TRUE else FALSE
        this->add_relation("max", "INTEGER INTEGER INTEGER");
        this->add_relation("maxR", "REAL REAL REAL");

        // X Y Z - if Z = round(X) to Y digits after dot then TRUE else FALSE
        this->add_relation("round", "REAL INTEGER REAL");
        //this->add_relation("round", "REAL INTEGER"); // if Z = round(X)

        // receives a logic value and returns it (used for passing logic values in rules)
        this->add_relation("logic_value", "LOGIC");
    }

    Inter4ql::logic_type::type Math::runPlugin(std::string relation, std::vector<Value*>* bindedParams) {
        Inter4ql::logic_type::type returnValue = Inter4ql::logic_type::UNKNOWN;

        if(relation == "lt" || relation == "ltR") {
                bool result = isless(bindedParams->at(0)->get_value_numeric(), bindedParams->at(1)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation == "le" || relation == "leR") {
                bool result = islessequal(bindedParams->at(0)->get_value_numeric(), bindedParams->at(1)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation == "gt" || relation == "gtR") {
                bool result = isgreater(bindedParams->at(0)->get_value_numeric(), bindedParams->at(1)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation == "ge" || relation == "geR") {
                bool result = isgreaterequal(bindedParams->at(0)->get_value_numeric(), bindedParams->at(1)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation.find("eq") != -1) {
                bool result = ((*bindedParams->at(0)) == (*bindedParams->at(1)));
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation.find("sum") != -1) {
                bool result = (bindedParams->at(0)->get_value_numeric() + bindedParams->at(1)->get_value_numeric() == bindedParams->at(2)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation.find("product") != -1) {
                bool result = (bindedParams->at(0)->get_value_numeric() * bindedParams->at(1)->get_value_numeric() == bindedParams->at(2)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation.find("quotient") != -1) {
                if(bindedParams->at(1)->get_value_numeric() == 0) {
                        returnValue = Inter4ql::logic_type::UNKNOWN;
                } else {
                        bool result = (bindedParams->at(0)->get_value_numeric() / bindedParams->at(1)->get_value_numeric() == bindedParams->at(2)->get_value_numeric());
                        if(result) {
                                returnValue = Inter4ql::logic_type::TRUE;
                        } else {
                                returnValue = Inter4ql::logic_type::FALSE;
                        }
                }
        }

        if(relation.find("min") != -1) {
                bool result = (std::min(bindedParams->at(0)->get_value_numeric(), bindedParams->at(1)->get_value_numeric()) == bindedParams->at(2)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation.find("max") != -1) {
                bool result = (std::max(bindedParams->at(0)->get_value_numeric(), bindedParams->at(1)->get_value_numeric()) == bindedParams->at(2)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation.find("round") != -1) {
                double shift = pow(10, bindedParams->at(1)->get_value_numeric());
                if(shift == 0)
                        return Inter4ql::logic_type::UNKNOWN;

                bool result = ((round(bindedParams->at(0)->get_value_numeric() * shift) / shift) == bindedParams->at(2)->get_value_numeric());
                if(result) {
                        returnValue = Inter4ql::logic_type::TRUE;
                } else {
                        returnValue = Inter4ql::logic_type::FALSE;
                }
        }

        if(relation.find("logic_value") != -1) {
        	int logic_value = bindedParams->at(0)->get_value_int();
        	returnValue = (Inter4ql::logic_type::type) logic_value;
        }

        return returnValue;
    }

	void Math::reason()
	{

	}

	Result Math::ask_for_result(Fact * term) const
	{
		auto return_val = runPlugin(term->get_relation(), term->get_params());
		Result ret;
		ret.add_element({}, return_val);
		return ret;
	}

    void Math::add_relation(std::string name, std::string interface) {

        std::vector<Domain *> *ii = new std::vector<Domain*> ();

           std::istringstream iss(interface);

            do
            {
                //TYPE = 1, VARIABLE = 2, ID = 3, STRING = 4, INTEGER = 5, REAL = 6, DATE = 7, DATE_TIME = 8, LOGIC = 9, INVALID = 10

                std::string sub;
                iss >> sub;

                if(sub == "LITERAL") {
                        ii->push_back(new Domain(variable_type::ID));
                        continue;
                }
                if(sub == "STRING") {
                        ii->push_back(new Domain(variable_type::STRING));
                        continue;
                }
                if(sub == "INTEGER") {
                        ii->push_back(new Domain(variable_type::INTEGER));
                        continue;
                }
                if(sub == "REAL") {
                        ii->push_back(new Domain(variable_type::REAL));
                        continue;
                }
                if(sub == "DATE") {
                        ii->push_back(new Domain(variable_type::DATE));
                        continue;
                }
                if(sub == "DATE_TIME") {
                        ii->push_back(new Domain(variable_type::DATE_TIME));
                        continue;
                }
                if(sub == "LOGIC") {
                        ii->push_back(new Domain(variable_type::LOGIC));
                        continue;
                }
            } while (iss);

        this->relations->push_back(new Relation(name, ii, "math"));
    }


}
